from __future__ import annotations

from tcsoa.gen.Query._2012_10.Finder import SearchResponse, SearchInput
from tcsoa.utils import TcService


class FinderService(TcService):

    @classmethod
    def performSearch(cls, searchInput: SearchInput) -> SearchResponse:
        """
        This operation routes search request to a specific provider specified as providerName in the searchInput
        structures. 
        A framework will allow custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering.  Such provider can be a User Inbox retriever, or Full Text searcher. The new data
        provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The implementation
        is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject
        ---- Fnd0BaseProvider (foundation template)
        -------- Fnd0GetChildrenProvider(foundation template)
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template)
        -------- Aos0TestProvider (aosinternal template)
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        This operation allows user to send the search input, filter values, and sorting data.  These input values will
        then be passed to the fnd0performSearch operation input values, which then use to collect, sort, and filter its
        results.
        The first two input parameters are important.  The first input is provider name.  This is a string to represent
        the type name of RuntimeBusinessObject to which this request should be routed to.  If the template that
        contains the class is not installed, a partial error is returned in the service data. The second input is the
        search input map.  The key will be different per each provider.  For example, for Full Text searcher, the input
        key would be searchString.  For User Inbox retriever, the input key would be Inbox Type.  The fnd0performSearch
        implementation for each provider shall take into account on the key name as it is used to store the values in
        OperationInput object.
        """
        return cls.execute_soa_method(
            method_name='performSearch',
            library='Query',
            service_date='2012_10',
            service_name='Finder',
            params={'searchInput': searchInput},
            response_cls=SearchResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PartDesOccAlignmentData(TcBaseObj):
    """
    The input parameter for performing Alignment or Unalignments
    
    :var partOccurrence: Awb0Element object representing the Part Occurrence
    :var designOccurrence: Awb0Element object representing the Design Occurrence
    :var partContext: Awb0Element object representing the Part context. The value can be NULL.
    :var designContext: Awb0Element object representing the Design context. The value can be NULL.
    """
    partOccurrence: Awb0Element = None
    designOccurrence: Awb0Element = None
    partContext: Awb0Element = None
    designContext: Awb0Element = None



from tcsoa.gen.Internal_ProductConfiguratorAw._2017_12.services import ConfiguratorManagementService as imp0
from tcsoa.gen.Internal_ProductConfiguratorAw._2020_05.services import ConfiguratorManagementService as imp1
from tcsoa.gen.Internal_ProductConfiguratorAw._2018_05.services import ConfiguratorManagementService as imp2
from tcsoa.gen.Internal_ProductConfiguratorAw._2020_12.services import ConfiguratorManagementService as imp3
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):
    createCustomVariantRule = imp0.createCustomVariantRule
    getConfiguratorDataHierarchy = imp1.getConfiguratorDataHierarchy
    getFilterPanelData = imp2.getFilterPanelData
    getVariantConfigurationData = imp0.getVariantConfigurationData
    getVariantExpressionData = imp2.getVariantExpressionData
    getVariantExpressionData2 = imp3.getVariantExpressionData2
    setVariantExpressionData = imp2.setVariantExpressionData
    setVariantExpressionData2 = imp3.setVariantExpressionData2
    validateProductConfigurations = imp2.validateProductConfigurations
    validateProductConfigurations2 = imp3.validateProductConfigurations2
    variantConfigurationView = imp2.variantConfigurationView
    variantConfigurationView2 = imp3.variantConfigurationView2



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, Fnd0MarkupChange, Fnd0Markup
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MassUpdateAffectedInput(TcBaseObj):
    """
    This structure provides a set of input values for the massGetAffectedParents operation.
    
    :var target: Item Revision to base the where used search on.
    :var change: The change item revision object to record mass update results in.
    :var operation: Operation type used to help identify type of filtering to use when searching for impacted parent
    parts.
    """
    target: ItemRevision = None
    change: ItemRevision = None
    operation: int = 0


@dataclass
class MassUpdateAffectedOutput(TcBaseObj):
    """
    This structure provides a set of output values for the massGetAfectedParents operation.
    
    :var parent: This is the impacted parent part that references the target part.
    :var selectable: This indicates if the impacted parent part is selectable for mass update or not.
    :var selectableComment: This text informs why the impacted parent part is not selectable.
    """
    parent: ItemRevision = None
    selectable: bool = False
    selectableComment: str = ''


@dataclass
class MassUpdateAffectedResponse(TcBaseObj):
    """
    This structure contains all the results from the massGetAffectedParents operation.
    
    :var massUpdateAffectedOutputs: This is a vector of MassUpdateAffectedOutput structures.
    :var serviceData: Standard service data to handle partial errors.
    """
    massUpdateAffectedOutputs: List[MassUpdateAffectedOutput] = ()
    serviceData: ServiceData = None


@dataclass
class MassUpdateExecuteECNoutput(TcBaseObj):
    """
    This structure provides a set of output values for the massUpdateExecutionECN operation.
    
    :var changeItemRev: Change item revision used for mass update execution on the ECN change management process.
    :var markup: Markup on ChangeItemRev that was processed during the ECN CM process.
    :var markupChanges: This is a vector of markup changes that was processed during the ECN CM process.
    """
    changeItemRev: ItemRevision = None
    markup: Fnd0Markup = None
    markupChanges: List[Fnd0MarkupChange] = ()


@dataclass
class MassUpdateExecuteECNresponse(TcBaseObj):
    """
    This structure provides output for the massUpdateExecutionECN operation.
    
    :var outputs: This is a vector of MassUpdateExecuteECNoutput structures.
    :var serviceData: Standard service data for partial errors.
    """
    outputs: List[MassUpdateExecuteECNoutput] = ()
    serviceData: ServiceData = None


@dataclass
class MassUpdateExecuteECRinput(TcBaseObj):
    """
    This structure provides a set of input values for the massUpdateExecutionECR operation.
    
    :var executionMode: This defines the ECR execution being performed 1=one time execute, 2=save as markup and
    3=remove markup.
    :var operation: The mass update operation type being executed.
    :var target: Part that is being replaced or removed.
    :var addToProblem: Add the target part to the problem folder.
    :var addToSolution: Add the newItem to solution folder.
    :var newItem: The part that is doing the replacing or being added.
    :var change: The change item revision object to record mass update results in.
    :var markupChange: The markup change to remove from the markup and the change item revision object impacted folder.
    :var parents: A vector of impacted parent parts that reference the target part.
    :var sParents: A vector of bools that indicate if the impacted parent parts are selectable or not for mass update.
    """
    executionMode: int = 0
    operation: int = 0
    target: ItemRevision = None
    addToProblem: bool = False
    addToSolution: bool = False
    newItem: ItemRevision = None
    change: ItemRevision = None
    markupChange: Fnd0MarkupChange = None
    parents: List[ItemRevision] = ()
    sParents: List[bool] = ()


@dataclass
class MassUpdateExecuteECRoutput(TcBaseObj):
    """
    This structure provides a set of output values for the massUpdateExecutionECR operation.
    
    :var parent: The impacted parent part that references the target part.
    :var status: This indicates if the impacted parent part was successful or not during the massUpdateExecutionECR
    operation.
    :var statusComment: Detailed status text on why the impacted parent part succeded or failed to process correctly.
    """
    parent: ItemRevision = None
    status: int = 0
    statusComment: str = ''


@dataclass
class MassUpdateExecuteECRresponse(TcBaseObj):
    """
    This structure provides output for the massUpdateExecutionECR operation.
    
    :var outputs: This is a vector of MassUpdateExecuteECRoutput structures.
    :var serviceData: Standard service data for partial errors.
    """
    outputs: List[MassUpdateExecuteECRoutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2013_05.StructureExpansionLite import ExpansionResponse, PartialExpansionControls, Controls, LineAndPaths
from tcsoa.gen.BusinessObjects import RuntimeBusinessObject, Fnd0BOMLineLite
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureExpansionLiteService(TcService):

    @classmethod
    def getUndelivered(cls, undeliveredLineUids: List[str], controls: Controls) -> ExpansionResponse:
        """
        This operation returns Fnd0BomLineLite as objects for input UIDs of Fnd0BomLineLite. 
        It may be required to invoke this operation more than once as the number of Fnd0BomLineLite returned depends on
        outputPageSize specified in 'Controls'. The 'RelationAndTypesCriteria' of 'Controls' is used to retrieve
        Dataset, named references objects with FMS tickets when named reference object is file.
        
        
        Use cases:
        This operation helps while working with large structure. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes expandNext with top BOMLine to get next set of configured children as Fnd0BomLineLite objects
        4. Client checks if there are any undelivered Fnd0BomLineLite objects in 'ExpansionResponse'
        5. Client invokes getUndelivered operation to get next set of undelivered Fnd0BomLineLites along with Dataset
        6. Client uses configured structure and Dataset to render the product
        7. Client closes the BOMWindow to release resources
        """
        return cls.execute_soa_method(
            method_name='getUndelivered',
            library='Internal-StructureManagement',
            service_date='2013_05',
            service_name='StructureExpansionLite',
            params={'undeliveredLineUids': undeliveredLineUids, 'controls': controls},
            response_cls=ExpansionResponse,
        )

    @classmethod
    def unloadBelow(cls, lines: List[Fnd0BOMLineLite]) -> ServiceData:
        """
        This operation destroys children of input Fnd0BomLineLite from Teamcenter server. 
        
        
        
        Use cases:
        This operation is useful to release resources after rendering product.
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes expandNext with top BOMLine to get next set of configured children as Fnd0BomLineLite objects
        4. Client invokes expandNext recursively till structure is fully expanded
        5. Client uses configured structure and Dataset to render the product
        6. Client releases resource on Teamcenter server by calling unload operation
        """
        return cls.execute_soa_method(
            method_name='unloadBelow',
            library='Internal-StructureManagement',
            service_date='2013_05',
            service_name='StructureExpansionLite',
            params={'lines': lines},
            response_cls=ServiceData,
        )

    @classmethod
    def expandBasedOnOccurrenceList(cls, linePaths: List[LineAndPaths], controls: PartialExpansionControls) -> ExpansionResponse:
        """
        The operation performs partial expansion for the given 'LineAndPaths'. This operation also returns Dataset(s),
        named reference objects with FMS tickets which are requested through 'RelationAndTypesCriteria' in
        'PartialExpansionControls' for the paritally expanded child Fnd0BOMLineLite(s).
        
        
        
        
        Use cases:
        This operation is suited to get partially expanded configured structure and related Dataset for read-only
        scenarios, e.g. Visualization. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine
        2. Client optionally sets Variant Rule on BOMWindow
        3. Client pulls the occurrence_list from the VisStructureContext object
        3. Client invokes expandBasedOnOccurrenceList to get partially expanded configured structure
        4. Client uses configured structure and Dataset to render the product
        5. Client closes the BOMWindow to release resources
         
        """
        return cls.execute_soa_method(
            method_name='expandBasedOnOccurrenceList',
            library='Internal-StructureManagement',
            service_date='2013_05',
            service_name='StructureExpansionLite',
            params={'linePaths': linePaths, 'controls': controls},
            response_cls=ExpansionResponse,
        )

    @classmethod
    def expandNext(cls, linesToExpand: List[RuntimeBusinessObject], controls: Controls) -> ExpansionResponse:
        """
        The operation performs top down expansion for the given parent Fnd0BomLineLite or BOMLine. The level of
        expansion is based upon input 'Controls' such as maxLevel. The 'RelationAndTypesCriteria' of 'Controls' is used
        to retrieve Dataset, named references objects with FMS tickets when named reference object is file.
        
        
        
        
        
        
        Use cases:
        This operation is suited to get configured structure and related Dataset for read-only scenarios, e.g.
        Visualization. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes expandNext with top BOMLine to get next set of configured children as Fnd0BomLineLite objects
        4. Client invokes expandNext recursively till structure is fully expanded
        5. Client uses configured structure and Dataset to render the product
        6. Client closes the BOMWindow to release resources
        """
        return cls.execute_soa_method(
            method_name='expandNext',
            library='Internal-StructureManagement',
            service_date='2013_05',
            service_name='StructureExpansionLite',
            params={'linesToExpand': linesToExpand, 'controls': controls},
            response_cls=ExpansionResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetLocalizedPropertiesInfo(TcBaseObj):
    """
    A structure of desired business objects, property names.
    
    :var inputObject: The desired business object to retrieved localized property values.
    :var propertyNames: A list of property name string.
    If propertyNames is an empty array, will get all displayable localizable properties.
    """
    inputObject: BusinessObject = None
    propertyNames: List[str] = ()


@dataclass
class GetLocalizedPropertiesInput(TcBaseObj):
    """
    The input structure for a list of business object info structure and a list of language locales.
    
    :var info: A list of desired business objects, property names to retrieve those properties in.
    :var locales: A list of language locale string: en_US , de_DE, zh_CN, etc.
    If locales is an empty array, will get all supported language from the list of Java-standard formatted name of
    languages that are supported by the system and specified in the BMIDE Global Constant Fnd0SelectedLocales.
    """
    info: List[GetLocalizedPropertiesInfo] = ()
    locales: List[str] = ()


@dataclass
class GetLocalizedPropertiesResponse(TcBaseObj):
    """
    The response structure for the getLocalizedProperties service operation.
    
    :var propertiesInfo: A list of response structure that holds business object, the property information with name,
    values, statuses and locale of the property.
    :var fullTranslationStatuses: A list of all the full translation statuses.
    :var serviceData: The service data object.
    """
    propertiesInfo: List[ObjectLocalizedPropertiesInfo] = ()
    fullTranslationStatuses: List[TranslationStatusInfo] = ()
    serviceData: ServiceData = None


@dataclass
class LocalizedPropertyNameValuesInfo(TcBaseObj):
    """
    A structure that holds property name and a list of LocalizedValuesInfo structure.
    
    :var propertyName: The property name string.
    :var propertyValues: A structure list that holds property values, locale and status information.
    """
    propertyName: str = ''
    propertyValues: List[LocalizedValuesInfo] = ()


@dataclass
class LocalizedValuesInfo(TcBaseObj):
    """
    A structure that holds property values, statuses and locale information.
    
    :var values: A list of property values.
    :var statuses: A list of the localization status.
    The status must be one of the following values:
    "TranslationStatusApproved","TranslationStatusPending","TranslationStatusInReview","TranslationStatusInvalid".
    :var locale: The name of the locale. For example: de_DE, zh_CN, etc.
    """
    values: List[str] = ()
    statuses: List[str] = ()
    locale: str = ''


@dataclass
class ObjectLocalizedPropertiesInfo(TcBaseObj):
    """
    The input structure for a list of business object, the property information with name, values, statuses and locale
    of the property.
    
    :var inputObject: The business object.
    :var propertyNameValues: A list of structure that holds property name and a list of LocalizedPropertyNameValuesInfo
    structure.
    """
    inputObject: BusinessObject = None
    propertyNameValues: List[LocalizedPropertyNameValuesInfo] = ()


@dataclass
class TranslationStatusInfo(TcBaseObj):
    """
    A struct that holds status from TranslationStatus enumeration, status display name and description.
    
    :var status: status value from TranslationStatus enumeration.
    :var displayName: Display name of the status.
    :var description: Description of the status.
    """
    status: TranslationStatus = None
    displayName: str = ''
    description: str = ''


class TranslationStatus(Enum):
    """
    TranslationStatus enumeration associated with the status.
    """
    Master = 'TranslationStatusMaster'
    Approved = 'TranslationStatusApproved'
    Pending = 'TranslationStatusPending'
    InReview = 'TranslationStatusInReview'
    Invalid = 'TranslationStatusInvalid'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, BOMLine, Folder
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetRollupTemplateInput(TcBaseObj):
    """
    The input structure which contains data used to find RollupTemplates.
    
    :var clientId: A unique string used to identify the caller. It is simply returned on success, or used to label any
    errors.
    :var mode: The search mode string:
    "Mode=ALL"
    "Mode=USER_PREFERENCE"
    or
    "Mode=SPECIFIC:name:scope_context"
    """
    clientId: str = ''
    mode: str = ''


@dataclass
class ReviseRollupReportInput(TcBaseObj):
    """
    Input structure which defines data needed to revise rollup report.
    
    :var clientId: A unique string used to identify the caller.
    :var bomline: The root BOM line the report is based on.
    :var dataset: The existing rollup report you want to revise.
    :var rollupTemplate: The RollupTemplate on which to base the revised rollup report.
    """
    clientId: str = ''
    bomline: BOMLine = None
    dataset: Dataset = None
    rollupTemplate: BusinessObject = None


@dataclass
class RollupReportOutput(TcBaseObj):
    """
    A structure used by 'generateRollupReports' to return rollup reports.
    
    :var clientId: A unique string used to identify the caller.
    :var dataset: The generated rollup report Dataset tag.
    """
    clientId: str = ''
    dataset: Dataset = None


@dataclass
class RollupReportResponse(TcBaseObj):
    """
    Response from 'generateRollupReports' and 'reviseRollupReports' operations.
    
    :var output: A list of 'RollupReportOutput', which is a list of rollup report Dataset tags.
    :var serviceData: Status of the operation.
    """
    output: List[RollupReportOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RollupTemplateCalculation(TcBaseObj):
    """
    A structure used by the 'getRollupTemplateCalculations' operation to return rollup calculation templates.
    
    :var type: The rollup calculation template type: 'ROLLUPCalcSum', 'ROLLUPCalcCofm', 'ROLLUPCalcInertia', or
    'ROLLUPCalcRef'.
    :var calcTemplates: A list of rollup calculation templates of type "type".
    """
    type: ROLLUPCalculationType = None
    calcTemplates: List[BusinessObject] = ()


@dataclass
class RollupTemplateCalculationOutput(TcBaseObj):
    """
    A structure used by 'getRollupTemplateCalculations' to return RollupTemplates and their rollup calculation
    templates.
    
    :var rollupTemplate: A RollupTemplate tag.
    :var rollupCalcTemplates: The RollupTemplate's rollup calculation templates.
    """
    rollupTemplate: BusinessObject = None
    rollupCalcTemplates: List[RollupTemplateCalculation] = ()


@dataclass
class RollupTemplateCalculationResponse(TcBaseObj):
    """
    The returned structure from 'getRollupTemplateCalculations'.
    
    :var output: A list of 'RollupTemplateCalculationOutput', which is a list of rollup calculation template tags.
    :var serviceData: Status of the operation.
    """
    output: List[RollupTemplateCalculationOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RollupTemplateInput(TcBaseObj):
    """
    - Input structure which contains data needed to get calculation templates from RollupTemplates.
    - Input structure used to clone RollupTemplates.
    
    
    
    :var clientId: A unique string which identifies the caller. It is simply returned on success, or used to label any
    errors.
    :var rollupTemplate: The tag of a RollupTemplate.
    """
    clientId: str = ''
    rollupTemplate: BusinessObject = None


@dataclass
class RollupTemplateOutput(TcBaseObj):
    """
    Structure used by 'createRollupTemplates' to return created RollupTemplates.
    
    :var clientId: A unique string used to identify the caller. It is simply returned on success, or attached to any
    errors.
    :var rollupTemplates: A list of RollupTemplate tags.
    """
    clientId: str = ''
    rollupTemplates: List[BusinessObject] = ()


@dataclass
class RollupTemplateResponse(TcBaseObj):
    """
    The structure returned from 'createRollupTemplates' and 'getRollupTemplates'.
    
    :var output: A list of RollupTemplate tags.
    :var serviceData: Status of the operation.
    """
    output: List[RollupTemplateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateRollupCalculationInputArgument(TcBaseObj):
    """
    Input structure used by 'createRollupCalculationTemplates'.
    
    :var name: "Value" or "Asserted" depending on whether you are setting the property to use for the calculation or
    the property to use for asserted (use the value calculated by your CAD tool).
    :var input: The name of the property you want to use for the calculation.
    :var output: The label you want to use for the column in the rollup report. (Does not apply to Asserted, so use "".)
    """
    name: str = ''
    input: str = ''
    output: str = ''


@dataclass
class CreateRollupCalculationTemplateInput(TcBaseObj):
    """
    The input structure which defines data needed to create rollup calculation templates.
    
    :var clientId: A unique string used to identify the caller. It is simply returned on success, or used to ID any
    errors.
    :var type: An enumeration specifying the type of rollup calculation template to create: 'ROLLUPCalcSum',
    'ROLLUPCalcCofm', 'ROLLUPCalcInertia', or 'ROLLUPCalcRef'.
    :var parent: The tag of the RollupTemplate to attach the new rollup calculation template to.
    :var arguments: A vector of structures consisting of three strings - name, input, output.
    - Name is "Value" or "Asserted" depending on whether you are setting the property to use for the calculation or the
    property to use for asserted (use the value calculated by NX).
    - Input is the name of the property you want to use for the calculation.
    - Output is the label you want to use for the column in the rollup report. (Does not apply to Asserted, so use "".)
    
    """
    clientId: str = ''
    type: ROLLUPCalculationType = None
    parent: BusinessObject = None
    arguments: List[CreateRollupCalculationInputArgument] = ()


@dataclass
class CreateRollupCalculationTemplateOutput(TcBaseObj):
    """
    Output structure used by 'createRollupCalculationTemplates' to return created rollup calculation templates.
    
    :var clientId: A unique string used to identify the caller.
    :var type: The type of rollup calculation templates.
    :var calcTemplate: The created rollup calculation template.
    """
    clientId: str = ''
    type: ROLLUPCalculationType = None
    calcTemplate: BusinessObject = None


@dataclass
class CreateRollupCalculationTemplateResponse(TcBaseObj):
    """
    The returned structure from 'createRollupCalculationTemplates'.
    
    :var output: A list of 'CreateRollupCalculationTemplateOutput', which constists of:
    - The client ID string that was passed in.
    - The rollup calculation template type that was passed in.
    - The tag of the created rollup calculation template.
    
    
    :var serviceData: Status of the operation.
    """
    output: List[CreateRollupCalculationTemplateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateRollupReportInput(TcBaseObj):
    """
    Input structure which defines data needed to create a rollup report.
    
    :var clientId: A string which identifies the caller. Can be any unique string since it is simply returned on
    success.
    :var name: A string used to name the rollup report Dataset. If null, the ItemRevision name will be used.
    :var description: An optional string to describe the rollup report.
    :var bomline: The tag of the root BOM line on which to generate the rollup report.
    :var folder: Optional Teamcenter folder in which to put the rollup report. If null, the report will be attached to
    the ItemRevision corresponding to the specified root BOM line, otherwise on error it will be placed in the Newstuff
    folder.
    :var rollupTemplate: The RollupTemplate you want to use to generate the rollup report.
    """
    clientId: str = ''
    name: str = ''
    description: str = ''
    bomline: BOMLine = None
    folder: Folder = None
    rollupTemplate: BusinessObject = None


@dataclass
class CreateRollupTemplateInput(TcBaseObj):
    """
    Input structure which defines data needed to create RollupTemplates.
    
    :var clientId: A string which identifies the caller. The string can be anything since it is simply returned in
    'RollupTemplateOutput'.
    :var name: A string which will be used to name the RollupTemplate.
    :var desc: A description for the RollupTemplate.
    :var delimiter: The delimiter which will be used to separate columns in the rollup reports generated using this
    template.
    :var scopeContext: A string which gives context to the scope. For site scope - the site ID, for group scope - the
    group name, for user scope - the user name.
    :var scope: One of the following: 'ROLLUPScopeSite', 'ROLLUPScopeUser', 'ROLLUPScopeGroup'
    """
    clientId: str = ''
    name: str = ''
    desc: str = ''
    delimiter: str = ''
    scopeContext: str = ''
    scope: ROLLUPScopeType = None


class ROLLUPCalculationType(Enum):
    """
    Enumerator used to distinguish between different rollup calculation template types.
    """
    ROLLUPCalcInvalid = 'ROLLUPCalcInvalid'
    ROLLUPCalcSum = 'ROLLUPCalcSum'
    ROLLUPCalcCofm = 'ROLLUPCalcCofm'
    ROLLUPCalcInertia = 'ROLLUPCalcInertia'
    ROLLUPCalcRef = 'ROLLUPCalcRef'


class ROLLUPScopeType(Enum):
    """
    Enumerator used to distinguish between different rollup template scopes.
    """
    ROLLUPScopeInvalid = 'ROLLUPScopeInvalid'
    ROLLUPScopeSite = 'ROLLUPScopeSite'
    ROLLUPScopeUser = 'ROLLUPScopeUser'
    ROLLUPScopeGroup = 'ROLLUPScopeGroup'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0LicenseServer
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicenseServerInput(TcBaseObj):
    """
    The 'LicenseServerInput' structure contains information for properties such as license server name, host, port,
    protocol and failover servers, for a given license server.  A license server location is defined by its host and
    port.
    
    :var licenseServer: The license server object to be updated.
    :var licenseServerName: The input name of the license server.
    :var failoverLicenseServers: The list of license server objects used as failover license servers.
    :var hostName: The host name of the license server
    :var portNumber: The port number of the license server.  This is used in combination with the hostName to connect
    to the license server.
    :var protocol: The protocol used when connecting to the license server.  The protocol can only be "TCP/IP".
    """
    licenseServer: Fnd0LicenseServer = None
    licenseServerName: str = ''
    failoverLicenseServers: List[Fnd0LicenseServer] = ()
    hostName: str = ''
    portNumber: int = 0
    protocol: int = 0



from __future__ import annotations

from tcsoa.gen.Internal.Classification._2020_04.Classification import FindClassificationInfoRequest, FindClassificationInfoResponse
from typing import List
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def findClassificationInfo(cls, request: List[FindClassificationInfoRequest], options: int) -> FindClassificationInfoResponse:
        """
        Finds classification information based on the input criteria.
        
        This operation finds and returns all the classification objects for the given WorkspaceObject and
        classification class definition, its attribute definitions including the details of associated Classification
        KeyLOV (stxt).
        Return information such as class information, class images, relations can be optionally restricted using option
        parameter.
        
        Use cases:
        This operation could be used when user needs to find classification objects, based on workspace objects. Each
        time a WorkspaceObject is classified in a classification class a classification object is created. The
        operation findClassificationInfo can be used to get detailed information about the classification objects where
        these classification objects belong. After getting information about classification objects the user can choose
        to modify or delete the objects using saveClassificationObjects or deleteClassificationObjects service
        operations. 
        
        The operation findClassificationInfo can also be used when a user wants to find details of classification class
        definition, its attribute definitions including the details of associated classification KeyLOV (stxt) objects.
        This information can help user decide whether to classify a WorkspaceObject in particular classification
        classes.
        
        This operation also provides the option for user to exclude certain information such as class information,
        class images, relations that is not required for the use case.
        """
        return cls.execute_soa_method(
            method_name='findClassificationInfo',
            library='Internal-Classification',
            service_date='2020_04',
            service_name='Classification',
            params={'request': request, 'options': options},
            response_cls=FindClassificationInfoResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CheckOutForSignResponse(TcBaseObj):
    """
    This structure contains the list of true or false corresponding the input list of the Dataset objects and the list
    of partial errors.
    
    :var verdict: If the Dataset is digitally checked out, the corresponding verdict is true, otherwise false,
    corresponding one to one to input Dataset object.
    :var serviceData: The partial error list if there is any system errors.
    """
    verdict: List[bool] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import ScheduleTask, TaskDependency
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreatedDependenciesContainer(TcBaseObj):
    """
    The created dependencies
    
    :var createdDependencies: The new dependencies
    :var updatedTasks: The updated tasks
    :var serviceData: The service data
    """
    createdDependencies: List[TaskDependency] = ()
    updatedTasks: List[ScheduleTask] = ()
    serviceData: ServiceData = None


@dataclass
class DependencyCreateContainer(TcBaseObj):
    """
    Creates Dependencies between tasks in the same schedule.
    
    :var predTask: The predecessor task for the dependency
    :var succTask: The successor task for the dependency
    :var depType: The type of dependency: 0-FS, 1-FF, 2-SS, 3-SF
    :var lagTime: The lag time in minutes (480 ~ 1 day)
    """
    predTask: ScheduleTask = None
    succTask: ScheduleTask = None
    depType: int = 0
    lagTime: int = 0



from tcsoa.gen.Internal_Workflow._2017_11.services import WorkflowService as imp0
from tcsoa.gen.Internal_Workflow._2013_05.services import WorkflowService as imp1
from tcsoa.utils import TcService


class WorkflowService(TcService):
    createWorkflowAsync = imp0.createWorkflowAsync
    performActionAsync = imp1.performActionAsync



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetLogicalObjectInput2(TcBaseObj):
    """
    Holds the list of the root object instances and the logical type name to retrieve the matched logical object
    instances.
    
    :var rootObjects: A list of root object instances.
    :var loTypeName: Name of the logical object type for which the logical object instances are to be searched for. The
    logical object type name corresponds to the name of any subtype of Fnd0LogicalObject.
    """
    rootObjects: List[RootObject] = ()
    loTypeName: str = ''


@dataclass
class GetLogicalObjectOutput2(TcBaseObj):
    """
    Holds the list of the retrieved logical object instances and related classification objects. Size of
    logicalObjectsOutput is equal to the input rootObjects attribute of input structure GetLogicalObjectInput2.
    
    :var logicalObjectsOutput: A list of the retrieved logical object instances.
    """
    logicalObjectsOutput: List[LogicalObjectOutput2] = ()


@dataclass
class GetLogicalObjectResponse2(TcBaseObj):
    """
    Holds the list of the retrieved logical object along with classification object output response and partial errors.
    
    :var serviceData: Returned service data.
    :var loOutputs: A list of the retrieved logical object output response corresponding to a  logical object input.
    :var classificationObjectInfo: A map (string, classificationObjectInfo) of the ICO Ids and their class attribute
    info.
    """
    serviceData: ServiceData = None
    loOutputs: List[GetLogicalObjectOutput2] = ()
    classificationObjectInfo: ClassificationObjectInfoMap2 = None


@dataclass
class LogicalObjectInstance(TcBaseObj):
    """
    Holds the logical Object instance and its root and member's classification objects.
    
    :var loInstance: An instance of the Fnd0LogicalObject.
    :var memberClassificationObjects: Logical Object instance's root and member name and its classification object
    information.
    """
    loInstance: BusinessObject = None
    memberClassificationObjects: List[MemberClassificationObject2] = ()


@dataclass
class LogicalObjectOutput2(TcBaseObj):
    """
    This structure contains the logical object instance for a given root object given in input structure RootObject.
    
    :var root: An instance of the root Object. This is same object which is provided in input structure RootObject.
    :var logicalObjectInstances: A list of logical object instances. Only one logical object instance is returned for a
    given root object in the current implementation.
    """
    root: BusinessObject = None
    logicalObjectInstances: List[LogicalObjectInstance] = ()


@dataclass
class MemberClassificationObject2(TcBaseObj):
    """
    Contains member or root name and its classification object information.
    
    :var memberOrRootName: The internal name of the member or the root on the Logical Object Type.
    :var icoIDs: A list of ICO UIDs.
    """
    memberOrRootName: str = ''
    icoIDs: List[str] = ()


@dataclass
class AttributeValue2(TcBaseObj):
    """
    Structure for each classification object attribute value.
    
    :var attributeID: ID of the attribute.
    :var attributeName: Name of the attribute.
    :var attributeValues: The values of the attribute.
    :var unit: The unit of the attribute.
    """
    attributeID: int = 0
    attributeName: str = ''
    attributeValues: List[str] = ()
    unit: str = ''


@dataclass
class RootObject(TcBaseObj):
    """
    Holds the root object instance and its client id to retrieve the matched logical object instance.
    
    :var root: A root object instance. Persistent types are the only supported business objects for a root object.
    :var clientID: Client Id for the root object. This helps in uniqely mapping the input and output data.
    """
    root: BusinessObject = None
    clientID: str = ''


@dataclass
class ClassificationObjectInfo(TcBaseObj):
    """
    Structure containing classification object information.
    
    :var icoClassId: The classification class Id.
    :var icoID: The classification object Id.
    :var unitBase: A class unit system of measure.
    :var classifiedObjectUID: Classified WorkspaceObject object UID.
    :var icoAttributeValues: Attribute list of this classification object.
    :var icoClassName: The class name in which the ICO is classified.
    :var icoType: The type of the ICO. Possible values are:
    
    0 - default ICO,
    1 - generic and 
    2 - for a need ICO.
    
    Generic and Need type are added to support Piping and Instrumentation diagram application.
    Compared to default type, the visibility of these objects to the client is controlled by preferences
    (ICS_hidden_ico_types and ICS_support_generic_and_need).
    """
    icoClassId: str = ''
    icoID: str = ''
    unitBase: str = ''
    classifiedObjectUID: str = ''
    icoAttributeValues: List[AttributeValue2] = ()
    icoClassName: str = ''
    icoType: int = 0


"""
Contains classification object uid and its information.

&bull; Primary key : The classification object (ICO) UID uid. 
&bull; Value            : A reference of the structure containing additional ICO properties.
"""
ClassificationObjectInfoMap2 = Dict[str, ClassificationObjectInfo]



from __future__ import annotations

from tcsoa.gen.Notification._2014_10.SubscriptionManagement import SubscriptionsResponse, FindSubscriptionsCriteriaInput, SubscriptionInput, ModifySubscriptionInput, GetSubscriptionsCriteriaInput
from typing import List
from tcsoa.utils import TcService


class SubscriptionManagementService(TcService):

    @classmethod
    def getSubscriptions(cls, input: GetSubscriptionsCriteriaInput) -> SubscriptionsResponse:
        """
        This operation queries for all Subscriptions that meet the input criteria.  A variety of input parameters are
        allowed, including target, event type, subscriber and handler.  This is intended to return a larger number of
        Subscriptions based on a small set of input criteria.  Most of the parameters can be empty, and will not be
        considered as part of the query.
        
        Use cases:
        The client is searching for all Subscriptions created for a given user against a given object.
        """
        return cls.execute_soa_method(
            method_name='getSubscriptions',
            library='Notification',
            service_date='2014_10',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def modifySubscriptions(cls, inputs: List[ModifySubscriptionInput]) -> SubscriptionsResponse:
        """
        This operation modifies existing ImanSubscriptions.  All values specified in the inputs replace the values on
        the ImanSubscription in the database.  Each modification is verified to insure that a duplicate
        ImanSubscription is not created.
        
        Use cases:
        An existing Subscription needs to have one or more of its values modified.
        """
        return cls.execute_soa_method(
            method_name='modifySubscriptions',
            library='Notification',
            service_date='2014_10',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def createSubscriptions(cls, inputs: List[SubscriptionInput]) -> SubscriptionsResponse:
        """
        This operation creates multiple object or class based Subscriptions based on the provided inputs. A
        Subscription allows a specified piece of functionality to be executed whenever an event occurs on the target
        object. Inputs to this operation will allow the target object to be specified, along with the event to monitor.
        The functionality the execute is specified by an array of ImanActionHandler objects. These handlers will be
        executed in the order they are specified in the Subscription definition. The most common handler would be
        IMAN_Smtp_Mail_Notify, which is used to send an e-mail when the event is triggered. If this handler is
        specified, the notification object should also be included as input.
        
        This operation also creates class based Subscriptions. This is done by passing a type object as the target
        object.  Class based Subscriptions will apply to all objects of the specified type. These can be filtered down
        at execution time by specifying a condition or attribute criteria as input. The Condition or attribute criteria
        will be evaluated against the object on which the event occurred, and the handler will only be executed if the
        Condition evaluates to TRUE.  Conditions and attribute criteria only apply for class based Subscriptions. 
        Attribute criteria are made up of a list of attributes and the values to compare them against off the target
        object.  Each of these comparisons can be logically combined.
        
        
        Use cases:
        Object Based Subscription:
        The client creates an object based Subscription in order to trigger functionality when a specific event occurs
        against the object. An example of this would be subscribing to the "Check Out" event for a given object. Any
        time the object is checked out, the handler functionality will be executed. Most often, this functionality will
        send the given user an e-mail notification.
        
        Class Based Subscription:
        The client creates a class based Subscription in order to trigger functionality when a specific event occurs
        against any object of the given type. An example of this would be subscribing to the "Check Out" event for an
        ItemRevision objects. Any time the object is checked out, the handler functionality will be executed. Most
        often, this functionality will send the given user an e-mail notification.
        
        Class based subscriptions allow a Condition or attribute criteria to be specified as part of the definition.
        This allows the events to be filtered to a smaller level. Any time the event occurs against the given object of
        the specified type, the Condition will be evaluated against the object. If the Condition or attribute criteria
        evaluate to true, the event is posted and the handlers are executed. An example Condition would only evaluate
        to true if the object is part of a specific Project.  Attribute criteria can also be specified and will be
        evaluated when the Subscription is being processed.  If the attribute criteria values do not match, the
        Subscription will be rejected.  Both attribute criteria and a Condition can not be specified at the same time.
        """
        return cls.execute_soa_method(
            method_name='createSubscriptions',
            library='Notification',
            service_date='2014_10',
            service_name='SubscriptionManagement',
            params={'inputs': inputs},
            response_cls=SubscriptionsResponse,
        )

    @classmethod
    def findSubscriptions(cls, input: FindSubscriptionsCriteriaInput) -> SubscriptionsResponse:
        """
        This operation queries for all Subscriptions that meet the input criteria.  A variety of input parameters are
        allowed, including target, event type, subscriber, condition and handlers.
        
        Use cases:
        The client is searching for a previously created Subscription against a given object or type.
        
        Exceptions:
        >When the query operation could not be completed with the given criteria.  This can be caused due to invalid
        input objects.
        
        - 78020:    The operation could not be completed with the given input criteria.
        
        """
        return cls.execute_soa_method(
            method_name='findSubscriptions',
            library='Notification',
            service_date='2014_10',
            service_name='SubscriptionManagement',
            params={'input': input},
            response_cls=SubscriptionsResponse,
        )



from tcsoa.gen.Internal_Notification._2015_10.services import SubscriptionManagementService as imp0
from tcsoa.gen.Internal_Notification._2015_03.services import SubscriptionManagementService as imp1
from tcsoa.gen.Internal_Notification._2017_11.services import SubscriptionManagementService as imp2
from tcsoa.gen.Internal_Notification._2014_10.services import SubscriptionManagementService as imp3
from tcsoa.gen.Internal_Notification._2015_10.services import MessageManagementService as imp4
from tcsoa.gen.Internal_Notification._2015_03.services import MessageManagementService as imp5
from tcsoa.utils import TcService


class SubscriptionManagementService(TcService):
    createSubscriptionTemplates = imp0.createSubscriptionTemplates
    createSubscriptions = imp1.createSubscriptions
    findSubscriptions = imp1.findSubscriptions
    getApplicableSubscriptionTemplates = imp0.getApplicableSubscriptionTemplates
    getAssignEventData = imp1.getAssignEventData
    getAttachEventData = imp1.getAttachEventData
    getAttachEventData2 = imp0.getAttachEventData2
    getEventTypes2 = imp1.getEventTypes2
    getSubscribableProperties = imp2.getSubscribableProperties
    getSubscribableTypes = imp1.getSubscribableTypes
    getSubscriptionConditions = imp3.getSubscriptionConditions
    getSubscriptionInput = imp1.getSubscriptionInput
    modifySubscriptionTemplates = imp0.modifySubscriptionTemplates
    modifySubscriptions = imp1.modifySubscriptions
    saveAsSubscription = imp0.saveAsSubscription
    transferNotifications = imp1.transferNotifications
    unsubscribe = imp0.unsubscribe


class MessageManagementService(TcService):
    getUnreadMessages = imp4.getUnreadMessages
    getUserGroupAlias = imp5.getUserGroupAlias
    initializeAsynchMessaging = imp4.initializeAsynchMessaging



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RemoteCheckinFailure(TcBaseObj):
    """
    Object specific detailed failure information.
    
    :var failureObjects: A list of objects that have failed to check-in.
    :var failureCodes: A list of failure codes for this operation.
    :var failureStrings: A list of error strings corresponding to each error in failureCodes.
    """
    failureObjects: List[BusinessObject] = ()
    failureCodes: List[int] = ()
    failureStrings: List[str] = ()


@dataclass
class RemoteCheckinResponse(TcBaseObj):
    """
    The RemoteCheckinResponse returns detailed partial failure information in RemoteCheckinFailure structures along
    with a ServiceData.
    
    :var failureInfo: A list of RemoteCheckinFailure structures. Each structure contains failure information for a
    specific business object that was supplied.
    :var serviceData: The standard ServiceData return.
    """
    failureInfo: List[RemoteCheckinFailure] = ()
    serviceData: ServiceData = None


@dataclass
class RemoteCheckoutFailure(TcBaseObj):
    """
    Failure information  for specific objects that failed to remote check out.
    
    :var failureObjects: A list of objects that have failed to check out.
    :var failureCodes: A list of failure codes for the failureObjects.
    :var failureStrings: A list of error strings corresponding to each error in failureCodes.
    """
    failureObjects: List[BusinessObject] = ()
    failureCodes: List[int] = ()
    failureStrings: List[str] = ()


@dataclass
class RemoteCheckoutResponse(TcBaseObj):
    """
    The RemoteCheckoutResponse returns detailed partial failure information in RemoteCheckoutFailure structures along
    with a ServiceData.
    
    :var serviceData: The standard ServiceData return.
    :var failureInfos: A list of RemoteCheckoutFailure structures. Each structure contains failure information for a
    specific business object that was supplied.
    """
    serviceData: ServiceData = None
    failureInfos: List[RemoteCheckoutFailure] = ()


"""
Map of string array property names to values (string, vector).
"""
StringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ReleaseStatus, BOMLine, Effectivity
from tcsoa.gen.StructureManagement._2014_12.Effectivity import EffectivityInfoInput
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RemoveOccEffectivitiesInput(TcBaseObj):
    """
    The information required to remove effectivity objects from specified BOMLine objects.
    
    :var bomLine: The BOMLine whose effectivities are removed
    :var effectivityComponents: The effectivity objects to be removed
    """
    bomLine: BOMLine = None
    effectivityComponents: List[Effectivity] = ()


@dataclass
class RemoveRelStatusEffectivityInput(TcBaseObj):
    """
    The information required to remove Effectivity from a released status
    
    :var releaseStatus: The released status whose Effectivity is removed
    :var effectivityComponent: The effectivity object to be removed
    """
    releaseStatus: ReleaseStatus = None
    effectivityComponent: Effectivity = None


@dataclass
class EditOccEffectivityInput(TcBaseObj):
    """
    The information required to update occurrence effectivity for the  BOMLine
    
    :var bomLine: The BOMLine whose effectivity is updated
    :var effectivityComponent: The effectivity object to be updated
    :var effectivityInfoInput: A structure to hold effectivity info
    """
    bomLine: BOMLine = None
    effectivityComponent: Effectivity = None
    effectivityInfoInput: EffectivityInfoInput = None


@dataclass
class EditRelStatusEffectivityInput(TcBaseObj):
    """
    The information required to update effectivity on a released status
    
    :var releaseStatus: The released status whose effectivity is updated
    :var effectivityComponent: The effectivity object to be updated
    :var effectivityInfoInput: A structure to hold effectivity info
    """
    releaseStatus: ReleaseStatus = None
    effectivityComponent: Effectivity = None
    effectivityInfoInput: EffectivityInfoInput = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class BomLineInfo(TcBaseObj):
    """
    This structure holds the source and the target BOMLine objects.
    
    :var sourceLine: The source BOMLine.
    :var targetLine: The target BOMLine.
    """
    sourceLine: BOMLine = None
    targetLine: BOMLine = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AccessorInformation(TcBaseObj):
    """
    Information about the Accessor.
    
    :var accessorType: The type of the Accessor.
    :var accessorName: The name of the Accessor.
    :var accessorObject: Accessor Objects for which Accessor Information is requested. Accessor object can be instance
    of POM_accessor, AM_named_tag, Group, Role, User, RoleInOwningGroup, RoleInProject, RoleInProjectsOfObject,
    RoleInSchedule.
    """
    accessorType: str = ''
    accessorName: str = ''
    accessorObject: BusinessObject = None


@dataclass
class AccessorsInfoResponse(TcBaseObj):
    """
    This structure holds a list of Accessor Information for a given input accessors.
    
    :var accessorInformations: List of AccessorInformations for each given accessor.
    :var serviceData: Object that holds the partial errors occurred while deriving the accessor information of any of
    the given business objects.
    """
    accessorInformations: List[AccessorInformation] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Item, WorkspaceObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetEffectivityResponse(TcBaseObj):
    """
    The 'GetEffectivityResponse' structure represents the effectivity data returned by 'getEffectivity' operation.
    
    :var effectivityData: A list of BusinessObjects with effectivity data.
    :var serviceData: The service data contains partial errors, if there are any errors during the operation.
    """
    effectivityData: List[EffectivityInfo] = ()
    serviceData: ServiceData = None


@dataclass
class Effectivity(TcBaseObj):
    """
    The 'Effectivity' represents a validity range. The following constants have special meaning:
    - January 2, 1900 12:00 AM UTC: Open Start Date.
    - December 30, 9999 12:00 AM UTC: Open End Date.
    - December 26, 9999 12:00 AM UTC: Stock Out.
    - 1: Open Start Unit.
    - 2147483647: Open End Unit.
    - 2147483646: Stock Out.
    
    
    Effect in as well as effect out points may have NULL values, which indicate no value assigned:
    - -1: no unit value assigned.
    - NULL date: no date value assigned.
    
    
    
    :var unitIn: The unit at which validity range starts.
    :var unitOut: The unit at which validity range ends.
    :var dateIn: The date at which validity range starts.
    :var dateOut: The date at which validity range ends.
    :var endItem: The effectivity end Item. This is mandatory if unit range is specified.
    :var effectivityOptions: A list of 'EffectivityOption' for which the effectivity range is valid (optional). The
    valid effectivity options are available on the effectivity context associated with with product Item with relation
    Fnd0ProductEffConfigCxtRel.
    For example, effectivity context with item id EFFCTX001 is associated with product Item. It has option families
    defined with names Maturity Intent and Module Intent. Maturity Intent has values Design and Production. Module
    Intent has values Kit and Modkit.  The effectivity options can be specified as: 
    - 'familyNamespace'=EFFCTX001, 'familyName'=Maturity Intent, 'optionValue'=Design, 'opCode'=5
    - 'familyNamespace'=EFFCTX001, 'familyName'=Module Intent, 'optionValue'=Kit, 'opCode'=5
    
    """
    unitIn: int = 0
    unitOut: int = 0
    dateIn: datetime = None
    dateOut: datetime = None
    endItem: Item = None
    effectivityOptions: List[EffectivityOption] = ()


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    The 'EffectivityInfo' structure represents effectivity for a BusinessObject.
    
    :var clientId: A unique string to identify returned BusinessObject and partial errors associated with input.
    :var inputObject: The BusinessObject to which effectivity information to be associated. Supported types are:
    ItemRevision and Awb0Element.
    :var effectivity: A list of 'Effectivity' for an 'inputObject'.
    :var effectivityFormula: The effectivity formula for an 'inputObject'. Either 'effectivity' or 'effectivityFormula'
    can be specified while setting the effectivity. If both are specified, 'effectivity' is ignored.
    For example, for unit range 1 to 10 the corresponding formula is ([Teamcenter::]Unit >= 1 &amp; [Teamcenter::]Unit
    < 11 ) &amp; [Teamcenter::]EndItem = "HDD-Top"
    """
    clientId: str = ''
    inputObject: BusinessObject = None
    effectivity: List[Effectivity] = ()
    effectivityFormula: str = ''


@dataclass
class EffectivityInfoInput(TcBaseObj):
    """
    The 'EffectivityInfoInput' structure represents BusinessObject and its effectivity to be set.
    
    :var effectivityContext: The effectivity context to be associated with the 'inputObject' in 'effectivityData'
    (optional). If 'effectivityContext' is specified, a new Fnd0ProductEffConfigCxtRel relation is created between
    'inputObject' and 'effectivityContext'. Supported context types are: Item and ItemRevision. The context is
    considered only when 'inputObject' is of type ItemRevision and is ignored if 'inputObject' is of type Awb0Element.
    If 'NULL', effectivity context is not associated with 'inputObject'.
    :var effectivityData: A list of BusinessObjects with effectivity data.
    """
    effectivityContext: WorkspaceObject = None
    effectivityData: List[EffectivityInfo] = ()


@dataclass
class EffectivityOption(TcBaseObj):
    """
    The 'EffectivityOption' structure represents an additional effectivity validity along with date and unit
    effectivity ranges.
    
    :var familyNamespace: The namespace by which the effectivity option family is uniquely identified. The
    'familyNamespace' is the item id of effectivity context associated with product Item with relation
    Fnd0ProductEffConfigCxtRel.
    For example, effectivity context with item id EFFCTX001 is associated with product Item. It has option families
    defined with names Maturity Intent and Module Intent. Maturity Intent has values Design and Production. Module
    Intent has values Kit and Modkit. The 'familyNamespace' can be EFFCTX001.
    :var familyName: The name of the effectivity option family. The valid option family names are available on the
    effectivity context associated with with product Item with relation Fnd0ProductEffConfigCxtRel.
    For example, effectivity context with item id EFFCTX001 is associated with product Item. It has option families
    defined with names Maturity Intent and Module Intent. Maturity Intent has values Design and Production. Module
    Intent has values Kit and Modkit. The 'familyNamespace' is set as EFFCTX001. The 'familyName' can be either
    Maturity Intent or Module Intent.
    :var opCode: The operator to be used for the 'optionValue'. 
    The valid values are:
    - 5 &ndash; Equals
    - 6 &ndash; Not Equals
    
    
    :var optionValue: The value for the effectivity option. The valid option values for given 'familyNamespace'  and
    'familyName' are available on the effectivity context associated with product Item with relation
    Fnd0ProductEffConfigCxtRel.
    For example, effectivity context with item id EFFCTX001 is associated with product Item. It has option families
    defined with names Maturity Intent and Module Intent. Maturity Intent has values Design and Production. Module
    Intent has values Kit and Modkit. The 'familyNamespace' is set as EFFCTX001. The 'familyName' is set Maturity
    Intent. The 'optionValue' can be either Design or Production.
    """
    familyNamespace: str = ''
    familyName: str = ''
    opCode: int = 0
    optionValue: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2011_06.DebugMonitor import OperationalHealthData, ValidMetrics, MetricMetaData, MetricAttributes, AlertEventData, MetricConfigMap
from tcsoa.gen.Internal.DebugMonitor._2011_06.DebugLogging import LoggerLevel, SQLLogInfoResponse, TAOLogLevelResponse, CheckingLevelResponse, ReturnStatus, LoggerLevelResponse, ModuleStatus, ModuleStatusResponse, SQLLogInfo
from typing import List
from tcsoa.gen.Internal.DebugMonitor._2011_06.PerformanceMonitor import PerformanceInfoResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DebugLoggingService(TcService):

    @classmethod
    def getSQLLogging(cls) -> SQLLogInfoResponse:
        """
        Retrieve the SQL logging options in effect in the current Teamcenter session.
        """
        return cls.execute_soa_method(
            method_name='getSQLLogging',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=SQLLogInfoResponse,
        )

    @classmethod
    def getStatisticsStatus(cls) -> bool:
        """
        Return the status of whether journal summary statistics are being collected or not in the current Teamcenter
        session.
        """
        return cls.execute_soa_method(
            method_name='getStatisticsStatus',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=bool,
        )

    @classmethod
    def getSyslogFile(cls, size: int) -> str:
        """
        Retrieves the most recent content of the current session`s active Teamcenter system log file (.syslog) as a
        string.
        """
        return cls.execute_soa_method(
            method_name='getSyslogFile',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'size': size},
            response_cls=str,
        )

    @classmethod
    def getTAOLogLevel(cls) -> TAOLogLevelResponse:
        """
        This operation returns the current value of the TAO logging level for the server.
        A Teamcenter business logic server is a CORBA server, and TAO (The ACE ORB) is its open source CORBA
        implementation software.  TAO is capable of producing a log file, named for example
        'C:\TEMP\tcserver.exe16e001fa.orblog'.  The TAO logging level is an integer 0..10 that controls which type of
        information is logged to the file.  No logging is done at level 0, and maximum logging is done at level 10. 
        Specific details of particular levels can be found in TAO software documentation, and in the TAO open source
        code.
        Unless set otherwise, the log level is 0, and nothing is written to the file.  At the end of a successful
        session, the log file is automatically removed if it is empty.
        """
        return cls.execute_soa_method(
            method_name='getTAOLogLevel',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=TAOLogLevelResponse,
        )

    @classmethod
    def resetStatistics(cls) -> bool:
        """
        Resets the collected journal summary statistics to an initial zeroed out state.
        
        Use cases:
        The 'resetStatistics'  operation is used to reset the statistics at the current point of execution in order to
        start collecting fresh statistics from this point onwards.  To collect statistics for a segment of program
        execution, an administrator using the Server Manager Administrator interface will: 
        1.    Proceed to the start of the desired functionality to be measured.
        2.    'setStatistics(true)',  to start gathering statistics.
        3.    'resetStatistics ()', to clear any stale accumulated statistics.
        4.    Proceed through the functionality to be measured.
        5.    'setStatistics (false)', to stop gathering statistics.
        6.    'dumpStatistics ()', to capture the information in the system log.
        """
        return cls.execute_soa_method(
            method_name='resetStatistics',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=bool,
        )

    @classmethod
    def setCheckingLevel(cls, checkingLevel: int) -> bool:
        """
        Set the checking level for the current Teamcenter business logic server.  See getCheckingLevel.
        
        Exceptions:
        >214600   The checking level is not in the range 0 through 2.
        """
        return cls.execute_soa_method(
            method_name='setCheckingLevel',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'checkingLevel': checkingLevel},
            response_cls=bool,
        )

    @classmethod
    def setJournalEntries(cls, journal: bool) -> bool:
        """
        Set whether or not the Teamcenter session will write journal entries to the journal file (.jnl)
        """
        return cls.execute_soa_method(
            method_name='setJournalEntries',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'journal': journal},
            response_cls=bool,
        )

    @classmethod
    def setLoggerLevels(cls, loggerInput: List[LoggerLevel]) -> ServiceData:
        """
        Set the priority level of loggers in the business logic server of the current Teamcenter session.  If a logger
        node with the given name does not yet exist, it will be created.
        """
        return cls.execute_soa_method(
            method_name='setLoggerLevels',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'loggerInput': loggerInput},
            response_cls=ServiceData,
        )

    @classmethod
    def setModuleJournalStatus(cls, journalModule: List[ModuleStatus]) -> ServiceData:
        """
        Set the journaling status of modules.
        
        The journaling subsystem tracks the entry and exit of major Teamcenter ITK routine and other major operations. 
        It writes the routine name, entry input parameter values, exit parameters, and execution time to the journal
        (.jnl) file.  The journal subsystem can be activated for specific modules, and deactivated for others.
        
        If any modules are turned on by this operation, the journal subsystem will be automatically turned on to write
        entries to the journal file.
        """
        return cls.execute_soa_method(
            method_name='setModuleJournalStatus',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'journalModule': journalModule},
            response_cls=ServiceData,
        )

    @classmethod
    def setSQLLogging(cls, sqlLogInfo: SQLLogInfo) -> bool:
        """
        Set SQL Logging settings for the current Teamcenter session .  These control the information written to the
        system log for each executed SQL request to the database.
        """
        return cls.execute_soa_method(
            method_name='setSQLLogging',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'sqlLogInfo': sqlLogInfo},
            response_cls=bool,
        )

    @classmethod
    def setStatistics(cls, stats: bool) -> bool:
        """
        Set whether or not to accumulate summary journal statistics in the current Teamcenter session.
        """
        return cls.execute_soa_method(
            method_name='setStatistics',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'stats': stats},
            response_cls=bool,
        )

    @classmethod
    def setTAOLogLevel(cls, logLevel: int) -> bool:
        """
        Set the logging level for the TAO CORBA server code in the business logic server of the current Teamcenter
        session.  See the documentation of 'getTAOLogLevel' for additional details.
        """
        return cls.execute_soa_method(
            method_name='setTAOLogLevel',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'logLevel': logLevel},
            response_cls=bool,
        )

    @classmethod
    def dumpStatistics(cls) -> bool:
        """
        Write the journal summary statistics to the Teamcenter system log file (.syslog)
        """
        return cls.execute_soa_method(
            method_name='dumpStatistics',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=bool,
        )

    @classmethod
    def getCheckingLevel(cls) -> CheckingLevelResponse:
        """
        Get the checking level.  The checking level is used in low level libsyss routines to activate additional
        diagnostic and validity checking code.  It is an integer 0,1,or 2, where 0 indicates no extra checking, 1
        indicates moderate checking, and 2 indicates advanced checking.   At checking level 2, performance may be
        significantly degraded.
        """
        return cls.execute_soa_method(
            method_name='getCheckingLevel',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=CheckingLevelResponse,
        )

    @classmethod
    def getDebugStatus(cls) -> ReturnStatus:
        """
        Retrieves the values of the primary debug statuses from the current Teamcenter session in a single request. 
        These are the Checking Level, Journaling status, Statistics status, TAO log level, and SQL logging settings.
        """
        return cls.execute_soa_method(
            method_name='getDebugStatus',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=ReturnStatus,
        )

    @classmethod
    def getJournalFile(cls, size: int) -> str:
        """
        Retrieves the most recent content of the current session`s active Teamcenter system journal file (.jnl) as a
        string.
        """
        return cls.execute_soa_method(
            method_name='getJournalFile',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'size': size},
            response_cls=str,
        )

    @classmethod
    def getJournalingStatus(cls) -> bool:
        """
        Return the journaling state of the current Teamcenter session.
        """
        return cls.execute_soa_method(
            method_name='getJournalingStatus',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=bool,
        )

    @classmethod
    def getLoggerLevels(cls, nodeName: str) -> LoggerLevelResponse:
        """
        Retrieves the names and current priority level setting of loggers in the business logic server of the current
        Teamcenter session.   The name of a logger leaf node or an intermediate node in the logger tree is given, and
        the names and priority levels of each logger node in the given subtree are returned.
        """
        return cls.execute_soa_method(
            method_name='getLoggerLevels',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={'nodeName': nodeName},
            response_cls=LoggerLevelResponse,
        )

    @classmethod
    def getModuleJournalStatus(cls) -> ModuleStatusResponse:
        """
        Retrieve the names and journaling status of each defined journaling module. The journaling subsystem tracks the
        entry and exit of major Teamcenter ITK routines and other major operations.  It writes the routine name, entry
        input parameter values, exit parameters, and execution time to the journal (.jnl) file.  The journal subsystem
        can be activated for specific modules, and deactivated for others.
        """
        return cls.execute_soa_method(
            method_name='getModuleJournalStatus',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugLogging',
            params={},
            response_cls=ModuleStatusResponse,
        )


class DebugMonitorService(TcService):

    @classmethod
    def getTcServerMonitoringMode(cls) -> str:
        """
        The Teamcenter Server Monitoring System can operate in either of the three different monitoring modes Off,
        Collect or Alert. This method retrieves the current mode of operation. When in Off mode, the monitoring system
        does not record any monitoring data for the performance metrics. Since, there is no history maintained no
        alerts situation happens. In the  Collect mode,  measurement data is recorded but no alerts are generated when
        thresholds exceed. In the Alert mode, the measurement data is not only collected but alerts are generated when
        thresholds for any of the metrics is exceeded.
        
        Use cases:
        The caller of the operation may want to check the monitoring mode and change it later depending on its current
        value. For example, the user may want to switch off the notifications or alerts if he / she finds them enabled
        and set the system to Collect mode.
        """
        return cls.execute_soa_method(
            method_name='getTcServerMonitoringMode',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={},
            response_cls=str,
        )

    @classmethod
    def clearAllHistory(cls) -> bool:
        """
        The Teamcenter Server Health Monitoring System internally stores data pertinent to the performance metrics such
        as metric measurements and notifications issued by the Monitoring System. this operation deletes this data for
        every metric which is being monitored by the system.
        """
        return cls.execute_soa_method(
            method_name='clearAllHistory',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={},
            response_cls=bool,
        )

    @classmethod
    def clearHistory(cls, metricIds: List[str]) -> bool:
        """
        The Teamcenter Server Health Monitoring System internally stores data pertinent to the performance metrics such
        as metric measurements and notifications issued by the Monitoring System. This operation deletes data for the
        selected metrics.
        """
        return cls.execute_soa_method(
            method_name='clearHistory',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'metricIds': metricIds},
            response_cls=bool,
        )

    @classmethod
    def getValidMetrics(cls) -> ValidMetrics:
        """
        The Teamcenter Server Health Monitoring System internally maintains a list of metrics for which it can monitor
        performance data. This operation retrieves a list of metric IDs corresponding to these metrics.
        """
        return cls.execute_soa_method(
            method_name='getValidMetrics',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={},
            response_cls=ValidMetrics,
        )

    @classmethod
    def reloadConfiguration(cls) -> bool:
        """
        This operation refreshes the configuration information in the Teamcenter Server Health Monitoring System by
        fresh parsing of serverMonitorConfig.xml file in TC_DATA.
        
        Exceptions:
        >If the file containing configuration information for the Teamcenter Server Health Monitoring
        system(serverMonitorConfig.xml file in TC_DATA folder) is corrupt, it throws exception.
        """
        return cls.execute_soa_method(
            method_name='reloadConfiguration',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={},
            response_cls=bool,
        )

    @classmethod
    def setMetricConfig(cls, configs: MetricConfigMap) -> bool:
        """
        This operation sets the Teamcenter  Server Health Monitoring system's metric configuration.
        """
        return cls.execute_soa_method(
            method_name='setMetricConfig',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'configs': configs},
            response_cls=bool,
        )

    @classmethod
    def setTcServerMonitoringMode(cls, tcServerMonMode: str) -> bool:
        """
        The Teamcenter Server Monitoring System can operate in three different monitoring modes Off, Collect or Alert.
        This operation sets the current mode of operation. When in Off mode, the monitoring system does not record any
        monitoring data for the performance metrics. Since, there is no history maintained no alerts are generated. In
        the Collect mode,  measurement data is recorded but no alerts are generated even if thresholds are exceeded. In
        the Alert mode, the measurement data is not only collected but alerts are generated when threshold  for any of
        the metrics is exceeded. If an unsupported mode of monitoring is specified, the system is set to Off mode.
        """
        return cls.execute_soa_method(
            method_name='setTcServerMonitoringMode',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'tcServerMonMode': tcServerMonMode},
            response_cls=bool,
        )

    @classmethod
    def getAlertInformation(cls, metricIds: List[str]) -> AlertEventData:
        """
        When the measurement values show a deviation from the expected behavior of the Teamcenter Server, alerts are
        triggered in the Teamcenter Server Health Monitoring System if they are enabled. For every metric Id there is a
        threshold set which defines the expected behavior. When exceeded, it causes the alerts to be generated and
        relevant data is posted in some in-memory data structure. This operation allows the developer to fetch this
        alert data for the desired metrics.
        
        Use cases:
        The caller of this service operation may want to retrieve detailed information pertaining to generated alerts
        like the set of value(s) that caused the alerts, comparison between the threshold and the set of value(s) that
        caused the alert to occur etc. He/ she can use this operation for retrieving it.
        """
        return cls.execute_soa_method(
            method_name='getAlertInformation',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'metricIds': metricIds},
            response_cls=AlertEventData,
        )

    @classmethod
    def getMetricMetaData(cls, metricIds: List[str]) -> MetricMetaData:
        """
        This operation retrieves the Meta data for desired metrics.. The Meta data is the information that describes
        the characteristics of the metrics that are being monitored by the Teamcenter Server Health Monitoring System.
        If user passes invalid metricIds on the input, they are simply ignored.
        
        Use cases:
        The user may want to know about the characteristics of various metrics before fetching their actual measurement
        values in order to interpret the information correctly. For example units of measurement for a metric may help
        in knowing if the measured value for a time based metric is seconds or milliseconds.
        """
        return cls.execute_soa_method(
            method_name='getMetricMetaData',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'metricIds': metricIds},
            response_cls=MetricMetaData,
        )

    @classmethod
    def getOperationalHealthData(cls, metricAttributes: List[MetricAttributes]) -> OperationalHealthData:
        """
        This operation retrieves the measurement information for the desired performance metrics. The data is further
        restricted to only those metrics which were enabled for data collection in the system in the past. If there is,
        no previous measurement data collected for a particular metric id, an empty collection is returned for that
        metric. Some of the valid metric IDs are - POMRetries, Deadlocks, DBConnectionLosses, OmAllocations,
        OsBsmUndoPool and PomLocks. The entire list of valid metric IDs can be found in serverMonitorConfig.xml in
        TC_DATA folder. All invalid metric IDs are ignored.
        
        Use cases:
        User can fetch the measurement information based on performance metric IDs (unique identifiers for metrics) and
        the time interval. In order to interpret the measurement information correctly, user may want to call the
        getMetricMetaData operation prior to this operation. It would give information about the characteristics of
        measurements such as the units of measurement.
        """
        return cls.execute_soa_method(
            method_name='getOperationalHealthData',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='DebugMonitor',
            params={'metricAttributes': metricAttributes},
            response_cls=OperationalHealthData,
        )


class PerformanceMonitorService(TcService):

    @classmethod
    def clearPerformanceInformation(cls) -> ServiceData:
        """
        This operation clears the stored performance and resource utilization data on the server and frees any memory
        allocated for such data.
        """
        return cls.execute_soa_method(
            method_name='clearPerformanceInformation',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='PerformanceMonitor',
            params={},
            response_cls=ServiceData,
        )

    @classmethod
    def monitorPerformance(cls, modes: str) -> ServiceData:
        """
        This operations sets performance parameter collection mode. The modes argument can be set to the following
        values:
        - Off    Turn off all instrumentation.
        - SoaOnly    All stored performance data is returned to the caller via the  'getPerformanceInformation' service
        operation call.
        - ServiceDataOnly    All stored performance data is returned to the caller on each service operation calls via
        the 'Fnd0Profiler'  object added to the 'ServiceData' created list.
        - Both    All stored performance data is returned to the call on the new service operation call and any service
        operations returning 'ServiceData'.
        
        
        If the 'modes' value is an invalid mode then it will be discarded and the profiling mode will be set to its
        default value Off.
        """
        return cls.execute_soa_method(
            method_name='monitorPerformance',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='PerformanceMonitor',
            params={'modes': modes},
            response_cls=ServiceData,
        )

    @classmethod
    def getPerformanceInformation(cls, clear: bool) -> PerformanceInfoResponse:
        """
        This operation fetches the performance parameters from the Teamcenter server. If the clear flag is set the
        stored performance data list is cleared and memory is freed, otherwise the performance data is retained and
        used for subsequent calls.
        """
        return cls.execute_soa_method(
            method_name='getPerformanceInformation',
            library='Internal-DebugMonitor',
            service_date='2011_06',
            service_name='PerformanceMonitor',
            params={'clear': clear},
            response_cls=PerformanceInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item, ItemRevision, BOMLine
from tcsoa.gen.StructureManagement._2008_06.Structure import DefaultNaming, NewItemInfo
from enum import Enum
from typing import List
from tcsoa.gen.StructureManagement._2010_09.Structure import ProjectInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandOrUpdateDuplicateItemsOutput2(TcBaseObj):
    """
    The 'ExpandOrUpdateDuplicateItemsOutput2' represents the collection of all related data objects with the structure
    ItemRevision object(s) or part family masters ItemRevision object(s).
    
    :var object: The BOM structure ItemRevision or a part family master ItemRevision.
    :var relatedObjInfo: List of all dependent objects that are related to the object.
    """
    object: ItemRevision = None
    relatedObjInfo: List[RelatedItemsInfo2] = ()


@dataclass
class ExpandOrUpdateDuplicateItemsResponse2(TcBaseObj):
    """
    The 'ExpandOrUpdateDuplicateItemsResponse2' structure represents the results of the structure dependent data search.
    
    :var outputs: List of 'ExpandOrUpdateDuplicateItemsOutput2' which contains information about the BOM structure
    ItemRevision to its related data relationships.
    :var serviceData: The error information will be recorded here.
    """
    outputs: List[ExpandOrUpdateDuplicateItemsOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class RelatedItemsInfo2(TcBaseObj):
    """
    The RelatedItemsInfo represents the dependent data found and how it is related to a structure ItemRevision.
    RelatedItemsInfo also can represent a family table member to a family table master.
    
    :var relObj: An ItemRevision  - it is the related data.
    :var relationshipName: The GRM relationship used to find the related data.
    :var depType: The cad dependency types to use for the operation.
    :var mandatory: Is dependent type mandatory or not - If it is mandatory that cad dependent type will come in
    pre-checked and the user will not be able to de-select it.
    """
    relObj: ItemRevision = None
    relationshipName: str = ''
    depType: DependencyType3 = None
    mandatory: bool = False


@dataclass
class ValidateStructureItemIdsInfo3(TcBaseObj):
    """
    Input to ValidateStructureItemIds3. It contains top BOMLine, structure of oldItemComponent to
    'NewItemInfo'(NewItemId, NewItemName), default naming pattern and list of dependency types specified by the user.
    
    :var topLine: Top BOMLine or the selected BOMLine of the structure or sub structure to be cloned.
    :var clonedIdStructure: Structure of OldItem components to the 'NewItemInfo' struct that contains the proposed New
    ItemId and New ItemName
    :var defaultName: Pattern to form a new ItemId.
    :var options: The list of dependency types.
    :var projects: The list of user defined projects to which the duplicated Item objects will be added.
    """
    topLine: BOMLine = None
    clonedIdStructure: List[DuplicateIdStructure] = ()
    defaultName: DefaultNaming = None
    options: List[DependencyType3] = ()
    projects: ProjectInfo = None


@dataclass
class ValidateStructureItemIdsResponse3(TcBaseObj):
    """
    response from ValidateStructureItemIds, containing Structure of OldItem component, NewItemInfo, list of
    itemrevisions failed validation and Service Data object.
    
    :var clonedIdStructure: Structure of OldItem components to the Structure NewItemInfo that contains the proposed New
    ItemId and New ItemName.
    :var serviceData: 'ServiceData' object.  The error stack will contain those ItemRevision objects that failed
    cloning and the reason.
    """
    clonedIdStructure: List[DuplicateIdStructure] = ()
    serviceData: ServiceData = None


@dataclass
class ClosureRuleVariableInfo(TcBaseObj):
    """
    Contains variable name and value pair where the variable is a part of the ClosureRule. The values are to be
    replaced with variable during ClosureRule evaluation.
    
    :var name: Variable name used in ClosureRule.
    :var value: Value for the variable.
    """
    name: str = ''
    value: str = ''


@dataclass
class DuplicateIdStructure(TcBaseObj):
    """
    Structure of oldItem component and NewItemInfo structure.
    
    :var oldItemComponent: The original Item
    :var newItemInfo: The proposed new ItemId and ItemName
    """
    oldItemComponent: Item = None
    newItemInfo: NewItemInfo = None


@dataclass
class DuplicateInputInfo3(TcBaseObj):
    """
    Input for Duplicate SOA API
    
    :var topLine: The top BOMLine or selected BOMLine of the structure to be cloned.  The user can select a sub
    assembly from the original structure to clone.  This input cannot be NULL.  If it is the duplicate dialog will not
    come up at all.
    :var clonedIdStructure: A structure of oldItem which points to the NewItemInfo.  The newItemInfo is made up of the
    proposed new ItemId and the proposed new ItemName
    :var defaultName: The pattern to form the new ItemId
    :var renameCadFile: Whether to rename the CAD files or not
    :var options: The list of dependency types.  This defines the cad traversal.
    :var projects: The user selected projects to add the duplicated Item objects to
    :var duplicateOption: A bitmap for the duplicate option. 1 means smart selection. It can be used for future
    extension.
    """
    topLine: BOMLine = None
    clonedIdStructure: List[DuplicateIdStructure] = ()
    defaultName: DefaultNaming = None
    renameCadFile: bool = False
    options: List[DependencyType3] = ()
    projects: ProjectInfo = None
    duplicateOption: int = 0


class DependencyType3(Enum):
    """
    Dependency Types
    """
    NoDep = 'NoDep'
    Drawing = 'Drawing'
    PartFamilyMaster = 'PartFamilyMaster'
    PartFamilyMember = 'PartFamilyMember'
    Required = 'Required'
    AllDep = 'AllDep'
    ExcludeFromBom = 'ExcludeFromBom'


class UnloadType(Enum):
    """
    This is an enum which tells whether the unloaded BOM can be recovered or not.
    """
    unrecoverable = 'unrecoverable'
    recoverable = 'recoverable'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyMarkupData(TcBaseObj):
    """
    The parameter to apply Markup.
    
    :var element: Awb0Element object for which Markup is to be applied
    :var recursive: Flag that indicates whether to apply all Markups for all elements under the specified element.
    :var evaluate: Flag that indicates whether to check write access (true) or actually modify (false) the element.
    """
    element: Awb0Element = None
    recursive: bool = False
    evaluate: bool = False



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.UiConfig._2014_11.UiConfig import ResetUIConfigResponse, ResetUIConfigInput
from tcsoa.utils import TcService


class UiConfigService(TcService):

    @classmethod
    def resetUIConfigs(cls, resetUiConfigsIn: List[ResetUIConfigInput]) -> ResetUIConfigResponse:
        """
        If a client needs to reset the column and/or command information for one or more client scopes, they can use
        this operation with the scope of the login user to receive the new effective UI configuration.
        """
        return cls.execute_soa_method(
            method_name='resetUIConfigs',
            library='Internal-UiConfig',
            service_date='2014_11',
            service_name='UiConfig',
            params={'resetUiConfigsIn': resetUiConfigsIn},
            response_cls=ResetUIConfigResponse,
        )



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CompareResultsCursor(TcBaseObj):
    """
    CompareResultsCursor is a cursor that is returned for use in subsequent calls to get next page of results.
    
    :var startReached: If true, the first page of the results has been reached.
    :var endReached: If true, the last page of the results has been reached.
    :var startIndex: The Cursor start position for the result returned so far.
    :var endIndex: The Cursor end position for the result returned so far.
    :var pageSize: The maximum number of results that can be returned in one service call.
    :var isForward: If true scrolling is done in forward direction.
    """
    startReached: bool = False
    endReached: bool = False
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0
    isForward: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LaunchInfoInput(TcBaseObj):
    """
    The structure contains the needed information to retrieve the launch URL for a Teamcenter Dataset for access Office
    Online Server.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var obj: The Teamcenter BusinessObject. The BusinessObject need to resolve to an Office dataset type such as
    MSWordX.
    :var action: The WOPI action name. The action name is defined by WOPI protocol.
    :var extraInfo: A map (string, string) for extra name value pair information. This is intended for future use.
    """
    clientId: str = ''
    obj: BusinessObject = None
    action: str = ''
    extraInfo: KeyValueMap = None


@dataclass
class LaunchInfoOutput(TcBaseObj):
    """
    The structure contains the information needed to access the Dataset from the Office Online Server.
    
    :var clientId: The unmodified value from the LaunchInfoInput.clientId. This can be used by the caller to indentify
    this data structure with the source input data.
    :var oosUrlString: The launch URL to access the Dataset from the Office Online Server.
    :var accessToken: Teamcenter access token.
    :var accessTtl: Teamcenter access token time to live.
    """
    clientId: str = ''
    oosUrlString: str = ''
    accessToken: str = ''
    accessTtl: str = ''


@dataclass
class LaunchInfoOutputResponse(TcBaseObj):
    """
    The structure contains a list of LaunchInfoOutput and the Service Data.
    
    :var outputs: A list of LaunchInfoOutput.
    :var serviceData: The Service Data.
    """
    outputs: List[LaunchInfoOutput] = ()
    serviceData: ServiceData = None


"""
String to String Map. It is extra information for the request.
"""
KeyValueMap = Dict[str, str]



from __future__ import annotations

from enum import Enum


class WhatIfAnalysisOption(Enum):
    """
    Specifies the What-If analysis option.
    Start = Start What-If analysis. by locking the Schedule so that other users will not be able to update the Schedule
    in other sessions.
    SaveAndContinue = Save the changes made on the Schedule to the database and continue What-If analysis. Save What-If
    changes and continue.
    SaveAndExit = Save What-If changes and exit to the database and exit the What-If analysis mode by releasing the the
    lock on the Schedule so that other users can modify the Schedule.
    CancelAndExit = Cancel What-If analysis by discarding the changes and exit.
    """
    Start = 'Start'
    SaveAndContinue = 'SaveAndContinue'
    SaveAndExit = 'SaveAndExit'
    CancelAndExit = 'CancelAndExit'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from tcsoa.gen.Internal.Visualization._2008_06.StructureManagement import RelationAndTypesFilter, ExpandPSRelatedObjectInfo
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandPSData1(TcBaseObj):
    """
     Through this structure, the child BOMLine , the object of the BOMLine and the object attached to the bom line
    object are returned.
    
    :var bomLine: BOMLine object reference of the children
    :var objectOfBOMLine: Object that the child represents
    :var indexOfrelatedObjectsInfo: The index into 'ExpandPSFromOccurrenceListOutput'::relatedObjects array that has the
     information about the related object and relation type.
    """
    bomLine: BOMLine = None
    objectOfBOMLine: BusinessObject = None
    indexOfrelatedObjectsInfo: List[int] = ()


@dataclass
class ExpandPSFromOccurrenceListInfo(TcBaseObj):
    """
    Input structure that defines the occurrences which are to be expanded into the BOM window
    
    :var occurListClientId: Identifier that helps the client track the object(s)
    :var occurrenceChainsByParent:  List of occurrences, each occurrence being a list of strings. If the
    useClientIdAsKey is true
     and when two OccurrenceChainList with same clientId is specified the second OccurrenceChainList  element will not
    be processed if the first OccurrenceChainList resolution is successful.
    """
    occurListClientId: str = ''
    occurrenceChainsByParent: List[OccurrenceChainList] = ()


@dataclass
class ExpandPSFromOccurrenceListOutput(TcBaseObj):
    """
    This structure contains the client identifier and the list of 'OccurrenceListResults'.
    
    :var clientId: Identifier that helps the client track the object(s) created. This is the clientId specified  in the
    'ExpandPSFromOccurrenceListInfo'.
    :var occurrenceList: List of 'OccurrenceListResults' structs describing the occurrences that were expanded
    """
    clientId: str = ''
    occurrenceList: List[OccurrenceListResults] = ()


@dataclass
class ExpandPSFromOccurrenceListPref(TcBaseObj):
    """
    A structure that allows for filtering criteria to be specified.
    
    :var prefKeyValue: A map( key = string, value = string) in which the following preferences can be passed as
    KeyValue pairs
    wantDatasets - true/false , wantSiblings - true/false, wantPathToContextParentLine - true/false
    useClientIdAsKey - true/false, bomExpandExcludeFilter - None/ExcludeICHistory/ExcludeGDEs/ExcludeNonImanItemLines
    
    wantDatasets - When passed  as true  causes the item revision to expand further so that its Datasets can be found.
    
    wantSiblings - When  true expands one level the BOMLines that were expanded using occurrence list to  return the
    siblings.
    
    wantPathToContextParentLine - When true returns the path upto the context BOMLine when the occurrence list is
    expanded using bl_abs_occ_id or bl_absocc_uid_in_topline_context. 
    
    useClientIdAsKey - When true  indicates that the clientId is to be used as an identifier to consider two different
    elements in the occurrence list as same. In a list containing one or more occurrence list elements with same client
    id, if the first element resolves into a valid BOMLine then the rest of the occurrence list with that clientId is
    ignored for processing. This is useful when multiple NGID are recorded in the part in different product context and
    the client sends in all of them to get any of them resolved.
    
    bomExpandExcludeFilter- The filter that is applied when the BOM is expanded to return the sibling
    lines(wantSiblings=true)
    
    :var info: List of the relation name and the Dataset types to return along with the children.If info.size() == 0
    and wantDatasets == true then all dataset types are to be considered.
    """
    prefKeyValue: StringKeyToStringValueMap = None
    info: List[RelationAndTypesFilter] = ()


@dataclass
class ExpandPSFromOccurrenceListResponse(TcBaseObj):
    """
    Response structure for the 'expandPSFromOccurrenceList'() operation.
    
    :var output: List of 'ExpandPSFromOccurrenceListOutput' structures containing the objects corresponding to the
    parent and the child BOMLines,their siblings and their related objects as well.
    :var serviceData: The service data contains the plain objects and the error stack, if any.
    :var relatedObjects: List of objects attached to BOMLine with given relation
    :var childrenOfParent: A map(Key = BOMLine, Value = Array of 'ExpandPSData1') that contains the expanded parent
    BOMLine as the key and 'ExpandPSData1' of its children as value. This is populated only when the wantSiblings is
    true.
    """
    output: List[ExpandPSFromOccurrenceListOutput] = ()
    serviceData: ServiceData = None
    relatedObjects: List[ExpandPSRelatedObjectInfo] = ()
    childrenOfParent: ParentLineToChildrenDataMap = None


@dataclass
class OccurrenceChain(TcBaseObj):
    """
    Input structure that defines a single occurrence uid chain from the topline (but not including topline) to the
    BOMLine that this chains represents.
    
    :var ngidClientId: Identifier that helps the client track the object(s) and helps to identify if two
    OccurrenceChain are same with respect to same parent.
    :var attributeNames: A list of 1 or more attribute names that corresponds to the values that make up the
    occurrenceChainStr
    :var occurrenceChainStr: List of occurrence UIDs that make up the chain of UIDs from the parent BOMLine (but not
    including the parent) down to the last occurrence in the chain
    """
    ngidClientId: str = ''
    attributeNames: List[str] = ()
    occurrenceChainStr: List[str] = ()


@dataclass
class OccurrenceChainList(TcBaseObj):
    """
    Input structure that defines the parent context and a list of occurrence chains to expand within that context.
    
    :var listClientId: Identifier that helps the client track the object(s) and helps to identify if two
    'OccurrenceChainList' are same with differnet NGID.
    :var parentBomLine: Object reference for the parent BOMLine of the occurrences that are described in occurrenceList.
    :var occurrenceList: List of occurrences, each occurrence being a list of strings. The occurrence is described in
    the context of the parent BOMLine. If the useClientIdAsKey is true and when two 'OccurrenceChain' with same
    clientId is specified the second 'OccurrenceChain' element will not be processed if the first 'OccurrenceChain'
    resolution is successful.
    """
    listClientId: str = ''
    parentBomLine: BOMLine = None
    occurrenceList: List[OccurrenceChain] = ()


@dataclass
class OccurrenceChainResult(TcBaseObj):
    """
    This structure contains the client identifier and the list of 'ExpandPSData1' structures for each occurrence in a
    given chain.
    
    :var clientId: Identifier that helps the client track the object(s) created. The client ID that was specified in
    the 
    'OccurrenceChain'.
    :var occurrenceChain: List of 'ExpandPSData1' structs describing the occurrences that were expanded in the given
    'OccurrenceChain'.
    """
    clientId: str = ''
    occurrenceChain: List[ExpandPSData1] = ()


@dataclass
class OccurrenceListResults(TcBaseObj):
    """
    This structure records the parent context and the occurrences expanded in that context.
    
    :var clientId: Identifier that helps the client track the object(s) created. The client ID that was specified in
    the 
    'OccurrenceChainList'.
    :var parent: 'ExpandPSData1' describing the parent BOM
    :var occurrenceList: List of 'OccurrenceChainResult' structs describing the occurrences that were expanded in the
    context of the given parent
    """
    clientId: str = ''
    parent: ExpandPSData1 = None
    occurrenceList: List[OccurrenceChainResult] = ()


"""
List of children for a given parent. This is returned when the wantSiblings is true.If the line is a leaf node then it would be found in the map.
"""
ParentLineToChildrenDataMap = Dict[BOMLine, List[ExpandPSData1]]


"""
This map holds the string value for a given string as key
"""
StringKeyToStringValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.ImportExport._2011_06.FileImportExport import ImportExportOptions
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportPackingInfo(TcBaseObj):
    """
    The structure contains a primary business object and a list of secondary business objects.
    
    :var primaryObj: The primary business object. The primary business object can be of any type.
    :var secondaryObjs: A list of secondary business objects. The secondary business objects can be of any type.
    """
    primaryObj: BusinessObject = None
    secondaryObjs: List[BusinessObject] = ()


@dataclass
class ExportToApplicationInputData3(TcBaseObj):
    """
    The ExportToApplicationInputData3 structure represents all of the data necessary to export selected objects to
    specific application like MSExcel.
    
    :var exportPackingInfos: A list of ExportPackingInfo.
    :var columnAttributes: The column attributes to export.
    :var applicationFormat: The application format such as "MSExcel" and "MSExcelLive".
    Supported application formats for this operation:
    - MSExcel    This format is used for exporting Teamcenter objects to static MSExcel application. 
    - MSExcelLive    This format is used for exporting Teamcenter objects to Live MSExcel application.
    
    
    :var exportOptions: List of ImportExportOptions to be used during the export process.
    """
    exportPackingInfos: List[ExportPackingInfo] = ()
    columnAttributes: List[str] = ()
    applicationFormat: str = ''
    exportOptions: List[ImportExportOptions] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Vendormanagement._2008_06.VendorManagement import GetVPSRConditionsResponse
from tcsoa.utils import TcService


class VendorManagementService(TcService):

    @classmethod
    def getVPSRConditions(cls) -> GetVPSRConditionsResponse:
        """
        This internal service operation gets a list of conditions to use for VendorPart Selection Rule. The Response
        structure returned contains a list of condition names and their expressions.
        
        Use cases:
        In a typical use case, there is a dialogue to be displayed which will let user select one of the VendorPart
        Selection Rule conditions.
        Once the dialogue is created, it may be populated using this operation.
        """
        return cls.execute_soa_method(
            method_name='getVPSRConditions',
            library='Internal-Vendormanagement',
            service_date='2008_06',
            service_name='VendorManagement',
            params={},
            response_cls=GetVPSRConditionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ConfigurationContext
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InputObjectData(TcBaseObj):
    """
    Structure containing the input object on which tool needs to be launched and the configuration information which
    needs to be applied in case the input object is of type ItemRevision.
    
    :var object: The business object on which configured tool needs to be launched.
    :var configCntx: Object holding the configuration information that needs to be applied on the input object if it is
    of type ItemRevision.
    """
    object: BusinessObject = None
    configCntx: ConfigurationContext = None


@dataclass
class InputObjectsStructure2(TcBaseObj):
    """
    Structure containing selected input object data on which pre-configured simulation tool needs to be launched.
    
    :var inputObjectDataVec: Array of InputObjectData on which pre-configured simulation tool needs to be launched.
    """
    inputObjectDataVec: List[InputObjectData] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ClassificationObject(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObject: The Classification object. If this is NULLTAG; a new classification object will be created
    otherwise existing classification object represented by clsObject will be updated.
    :var workspaceObject: The WorkspaceObject (WSO) that is associated by this Classification object. If this is
    NULLTAG, then a standalone classification object is created or updated. Allowed WSO types will be defined by the
    preference 'ICS_allowable_types'.
    :var properties: List of properties containing attribute Ids and their values.
    """
    clsObject: BusinessObject = None
    workspaceObject: WorkspaceObject = None
    properties: List[ClassificationProperty] = ()


@dataclass
class ClassificationProperty(TcBaseObj):
    """
    Structure representing Classification Property which holds attribute ids and their values.
    
    :var attributeId: The unique identifier of classification attribute or internal identifier for classification
    header properties like class Id, unit system etc.
    :var values: A list of values for this attribute in the context of a Classification object.
    For regular properties it's just one value. In case of VLA (variable length array) properties each value has its
    own entry.
    """
    attributeId: int = 0
    values: List[str] = ()


@dataclass
class CreateOrUpdateClsObjectsResponse(TcBaseObj):
    """
    Holds the classification objects returned by the createOrUpdateClassificationObjects operation.
    
    :var clsObjects: A list of created or updated Classification objects.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    clsObjects: List[ClassificationObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetReservationHistoryResponse(TcBaseObj):
    """
    GetReservationHistoryResponse
    
    :var histories: Reservation history.
    :var serviceData: Objects which are queried successfully are added to the ServiceData plain list.
    """
    histories: List[ReservationHistory] = ()
    serviceData: ServiceData = None


@dataclass
class ReservationHistory(TcBaseObj):
    """
    Reservation history.
    
    :var object: Object of reservation history.
    :var events: Sequence of history events, earliest first.
    """
    object: BusinessObject = None
    events: List[ReservationHistoryEvent] = ()


@dataclass
class ReservationHistoryEvent(TcBaseObj):
    """
    Single event in reservation history.
    
    :var dateTime: Date and time of event.
    :var user: User name.
    :var activity: Event type.  "Check-Out"/"Check-In"/"Cancel Check-Out".
    :var changeId: change id as provided during 'checkout' operation.
    :var comment: User comment
    """
    dateTime: str = ''
    user: str = ''
    activity: str = ''
    changeId: str = ''
    comment: str = ''



from __future__ import annotations

from tcsoa.gen.Core._2016_09.DataManagement import CreateAttachResponse
from tcsoa.gen.Core._2015_07.DataManagement import CreateIn2
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createAttachAndSubmitObjects(cls, inputs: List[CreateIn2]) -> CreateAttachResponse:
        """
        This is a generic operation for creation of business objects. This will also create any secondary (compound)
        objects that need to be created, assuming the CreateInput2 for the secondary object is represented in the
        nested CreateInput2 object. e.g. Item is primary object that also creates an  Item Master and ItemRevision.
        ItemRevision in turn creates ItemRevision Master. The input for all these levels is passed in through the
        recursive CreateInput2 object.
        
        This operation also performs following tasks:
        - A list of file names can be passed in through the dataToBeRelated input of the CreateIn2 input object, and
        Dataset objects for the files will be created and related to the created business object. The information
        needed to subsequently upload the file contents will be passed back to the user in the CreateAttachResponse
        object.
        - Submit the created business object to a workflow process. The input for creating the workflow process is
        passed in through the workflowData input of CreateIn2 object. 
        - Relate the created business object to the input target object.
        - If the created business object is under Item Revision Definition Configuration (IRDC) control, proposed file
        attachments are first evaluated against the object's IRDC settings to check if they are valid or not. Invalid
        files will be discarded. If the created business object is not under IRDC control, the files will simply be
        related to the created business object as Dataset objects.
        - Proposed file attachments to the created business object are specified as file names rather than unique ids
        of existing business objects. Empty Dataset objects are created and write tickets are fetched for the files
        that will be uploaded as named references.
        
        
        
        Use cases:
        Use Case 1: Create an Item object under IRDC control.
        
        The user wants to create and Item and its associated objects, and furthermore will set some of the create
        inputs so that the newly created object will be under Item Revision Definition Configuration (IRDC) control. No
        files or workflow inputs are provided.
        
        Use Case 2: Create an Item object under IRDC control and submit it to a workflow.
        
        As with Use Case 1, but the name of a workflow template is provided. After the Item object is created, it will
        be submitted to the specified workflow.
        
        Use Case 2: Create an Item object under IRDC control, attach files, and submit it to a workflow.
        
        As with Use Case 2, but a list of file names is provided. After the Item object is created, one Dataset object
        will be created for each file name and related to the ItemRevision. File tickets will be passed back to the
        user in the CreateAttachResponse object so that the files can be uploaded to the Dataset objects' named
        references. Then the newly created Item is submitted to the specified workflow.
        """
        return cls.execute_soa_method(
            method_name='createAttachAndSubmitObjects',
            library='Core',
            service_date='2016_09',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=CreateAttachResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MetaDataStampInputInfo(TcBaseObj):
    """
    This structure holds information about the objects to stamp along with the stamping context within which the
    stamped objects reside.
    
    :var metaStampInfoId: Client supplied unique identifier string that helps the client map the input to the
    MetaDataStampOutput.
    :var stampingContext: A stamping context BusinessObject within which the stamped objects reside.  The root line
    occurrence object of a structure, or an ItemRevision object that owns a DirectModel dataset are two examples of
    expected inputs for this argument, but intended to be general (could be any business object).
    :var stampedObjects: A list of any POM_objects on which the metadatastamp file is to be generated. POM_objects like
    Fnd0ModelViewProxy and Datasets like SnapshotViewData are two examples of expected input for this argument, but
    intended to be general ( could be any POM_object ).
    """
    metaStampInfoId: str = ''
    stampingContext: BusinessObject = None
    stampedObjects: List[StampedObjectInfo] = ()


@dataclass
class MetaDataStampOutput(TcBaseObj):
    """
    This structure contains the list of StampedObjectTicketInfo structures.
    
    :var stampObjectTickets: A list of StampedObjectTicketInfo structures.
    """
    stampObjectTickets: List[StampedObjectTicketInfo] = ()


@dataclass
class MetaDataStampOutputResponse(TcBaseObj):
    """
    This structure is used to return information from the getMetaDataStampWithContext operation. This structure holds
    information about serviceData and MetaDataStampOutputMap. The serviceData may contain information relevant to error
    messages, which are then associated to metaStampInfoId identifier and added to the error stack. The map,
    MetaDataStampOutputMap, contains information mapping the metaStampInfoId identifier to the MetaDataStampOutput
    struct, containing information about the tickets of the stamped object.
    
    :var metaDataStampOutputInfo: A map (string/MetaDataStampOutput) associating the unique metaStampInfoId identifier
    with the correct MetaDataStampOutput structure, containing tickets for the stamped objects for the given context
    object.
    :var serviceData: Any failure will be returned with error messages in the serviceData list of partial errors.
    """
    metaDataStampOutputInfo: MetaDataStampOutputMap = None
    serviceData: ServiceData = None


@dataclass
class StampedObjectInfo(TcBaseObj):
    """
    This structure holds information of the stamped object along with its unique identifier.
    
    :var uniqueStampObjectId: Client supplied unique identifier string that helps the client map the returned ticket to
    the stamped object.
    :var stampedObject: Any POM_object on which the metadatastamp file is to be generated. POM_objects like
    Fnd0ModelViewProxy and Datasets like SnapshotViewData are two examples of expected input for this argument, but
    intended to be general ( could be any POM_object ).
    """
    uniqueStampObjectId: str = ''
    stampedObject: POM_object = None


@dataclass
class StampedObjectTicketInfo(TcBaseObj):
    """
    This structure holds information of the ticket generated for the stamped object. Failures to get ticket for the
    stamped object maybe recorded as partial errors in the error stack and associated to uniqueStampObjectId identifier.
    
    :var uniqueStampObjectId: Client supplied unique identifier string that helps the client map the ticket to the
    stamped object.
    :var mdsFileTicket: The FMS transient read ticket generated for the stamped object. The file will be placed in the
    transient file volume and the caller will need to download it from there with the ticket sent by the service.
    """
    uniqueStampObjectId: str = ''
    mdsFileTicket: str = ''


"""
This structure maps the metaStampInfoId of the input to the MetaDataStampOutput structure.
"""
MetaDataStampOutputMap = Dict[str, MetaDataStampOutput]



from tcsoa.gen.Internal_EntCba._2019_12.services import AlignmentsService as imp0
from tcsoa.utils import TcService


class AlignmentsService(TcService):
    alignOccurrences = imp0.alignOccurrences
    unalignOccurrences = imp0.unalignOccurrences



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProcessAttrExchangeConfigInput(TcBaseObj):
    """
    This common structure is for the create, update or delete operations on the Attribute Exchange Configuration
    (Fnd0AttrExchangeConfig) object(s)
    
    
    
    This operation adds and/or update or delete a list of attribute exchange configurations that is referenced by
    Logical Object Type.  The attribute exchange configurations will have the attribute exchange directions (e.g
    Teamcenter to File or File to Teamcenter) corresponding to each of Logical Object Type&rsquo;s presented properties.
    
    :var logicalObjectType: An object of logical object type (Fnd0LogicalObject) to which attribute exchange
    configuration are to be added or updated, then relate to.
    :var propertyData: sdgfsdfg sdg sdgfsdfgsdf
    """
    logicalObjectType: BusinessObject = None
    propertyData: List[PropertyData] = ()


@dataclass
class PropertyData(TcBaseObj):
    """
    This property data structure of presented property and corresponding actions of create, update or delete to perform
    on the Fnd0AttrExchangeConfig object.
    
    :var presentedPropertyName: The Fnd0LogicalObjectType&rsquo;s presented property name.
    :var action: The action to perform on the Fnd0AttrExchangeConfig object.  Valid actions are 'add', 'update' or
    'delete'.
    """
    presentedPropertyName: str = ''
    action: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, ADA_License
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConfigurationContextChoice(TcBaseObj):
    """
    A strucure that contains information related to Ace Context.
    
    :var internalValue: Internal value of business object.
    :var displayValue: Display value of business object.
    :var isDefaultValue: If true, this instance of ConfigurationContextChoice should be set as default choice.
    """
    internalValue: str = ''
    displayValue: str = ''
    isDefaultValue: bool = False


@dataclass
class FilterInfo(TcBaseObj):
    """
    A structure containing license related information to filter licenses.
    
    :var licenseType: The type of ADA licenses to be filtered.
    :var licenseId: The ID of the license to be filtered.
    """
    licenseType: str = ''
    licenseId: str = ''


@dataclass
class LicenseInput(TcBaseObj):
    """
    Structure that holds the information required to get list of licenses.
    
    :var selectedObjects: The list of selected WorkspaceObject objects
    :var selectedLicenses: The list of licenses (ADA_License) which are already selected to attach with objects.
    :var filterInfo: The filter criteria based on license type and license ID.
    :var paginationInfo: The pagination criteria.
    :var isAceContext: If true, the user is in ActiveWorkspace Content context.
    :var isAssigned: If true only already attached licenses to objects will return in response. If false than attached
    and available both licenses will return in response.
    :var licenseParamsIn: A list of parameters for future use.
    """
    selectedObjects: List[WorkspaceObject] = ()
    selectedLicenses: List[ADA_License] = ()
    filterInfo: FilterInfo = None
    paginationInfo: PaginationInfo = None
    isAceContext: bool = False
    isAssigned: bool = False
    licenseParamsIn: List[str] = ()


@dataclass
class LicenseOutput(TcBaseObj):
    """
    Structure that holds the licenses object.
    
    :var commonAssignedLicenseList: A list of ADA_License objects which are assigned to the seletectd InputObect. If
    the InputObject is a single select, ead_paragraph will be include. If the InputObject is a multi-select,
    ead_paragraph will be blank.
    :var availableLicenseList: A list of ADA_License objects which are available to attach to the selected InputObject.
    :var allAssignedLicenseList: List of all licenses already selected for attach to selected objects.
    :var assignedLicenseTypes: A list of types of the assignedLicenseList.
    :var availableLicenseTypes: A list of types of the availableLicenseList.
    """
    commonAssignedLicenseList: List[ADA_License] = ()
    availableLicenseList: List[ADA_License] = ()
    allAssignedLicenseList: List[ADA_License] = ()
    assignedLicenseTypes: List[LicenseType] = ()
    availableLicenseTypes: List[LicenseType] = ()


@dataclass
class LicenseType(TcBaseObj):
    """
     structure that holds internal value and display value of ADA_License.
    
    :var internalValue: Internal value of ADA_License.
    :var displayValue: Display value of ADA_License.
    """
    internalValue: str = ''
    displayValue: str = ''


@dataclass
class LicensesWithTypesResponse(TcBaseObj):
    """
    A strucure that holds all license related and pagination related information.
    
    :var licensesOutput: A list of licenseOutputs objects.
    :var licenseOptions: A map (string, list of ConfigurationContextChoice) of ACE context atrributes with the list of
    ACE context atrtributes and possible values.
    :var totalFound: The total licenses found.
    :var endIndex: The end index of the ADA_LICENSE returned by the operation. This end index will be used by the
    client as start index of next iteration.
    :var totalLoaded: The total number of licenses loaded at a time.
    :var licenseParamsOut: A map (string, list of string) for future use. It will used to return values which passed in
    licenseParamsIn parameter of LicenseInputs structure.
    :var serviceData: The SOA framework object containing objects that were created, deleted or updated by the Service,
    plain objects and error information. For this service, all object are returned to the plain objects group. Error
    information will also be returned.
    """
    licensesOutput: LicenseOutput = None
    licenseOptions: LicenseOptions = None
    totalFound: int = 0
    endIndex: int = 0
    totalLoaded: int = 0
    licenseParamsOut: LicenseParamsOut = None
    serviceData: ServiceData = None


@dataclass
class PaginationInfo(TcBaseObj):
    """
    A structure containing pagination criteria.
    
    :var startIndex: The start index to return the licenses. Must be a non-negative value. Otherwise all data will be
    returned and the data will not  be paginated.
    :var maxToReturn: The maximum number of licenses to return. Must be a value greater than zero. Otherwise, all data
    will be returned and the data will not be paginated.
    """
    startIndex: int = 0
    maxToReturn: int = 0


"""
A map (string, list of ConfigurationContextChoice) of ACE context atrributes with the list of ACE context atrtributes and possible values.
"""
LicenseOptions = Dict[str, List[ConfigurationContextChoice]]


"""
A map (string, list of string) for future use. It will used to return values which passed in licenseParamsIn parameter of LicenseInputs structure.
"""
LicenseParamsOut = Dict[str, List[str]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Workflow._2020_01.Workflow import SupportedHandlerArgumentsInput, SupportedHandlerArgumentsReponse
from tcsoa.utils import TcService


class WorkflowService(TcService):

    @classmethod
    def getSupportedHandlerArguments(cls, input: List[SupportedHandlerArgumentsInput]) -> SupportedHandlerArgumentsReponse:
        """
        Get hints from <handler_name>.json file placed in TC_DATA/workflow_handlers location which is helpful for
        handler creation in Workflow Designer. The following hints are supported by this service operation.
        &bull;    Mandatory arguments.
        &bull;    Optional arguments.
        &bull;    Mutex arguments.
        &bull;    Dependent arguments.
        &bull;    Nullable arguments.
        &bull;    Required one of arguments.
        &bull;    And argument values which allow only particular set of values for eg: list of object types, status,
        relations, LOVs etc.
        
        For detailed information, please refer to Teamcenter Active Workspace Documentation link:
        https://docs.plm.automation.siemens.com/tdoc/aw/4.3/aw_html_collection#uid:xid1284832:index_Configuration:xid1567004:xid1760071
        This link points to Active Workspace documentation path ( applicable Active Workspace 4.3 onwards ): 
        Home > Deployment and Configuration > Configuration and Extensibility > Working with platform customizations >
        Enable your custom workflow handler in Active Workspace
        
        Use cases:
        Data provided by this operation will be used by Workflow Designer in Active Workspace to simplify handler
        configuration in templates.
        """
        return cls.execute_soa_method(
            method_name='getSupportedHandlerArguments',
            library='Workflow',
            service_date='2020_01',
            service_name='Workflow',
            params={'input': input},
            response_cls=SupportedHandlerArgumentsReponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Classification._2009_10.Classification import GetClassificationPropertiesResponse, GetClassificationHierarchiesResponse
from typing import List
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class ClassificationService(TcService):

    @classmethod
    def getClassificationHierarchies(cls, wsos: List[WorkspaceObject]) -> GetClassificationHierarchiesResponse:
        """
        Finds information on classification class hierarchies corresponding to all the classification classes where the
        specified workspace object(s) is classified. For each classification class hierarchy, the name & ID of classes
        in it are retrieved
        
        Use cases:
        User wants to retrieve information about class hierarchy of the classification class in which a workspace
        object (WSO) is classified. This operation is similar to 'getClassificationProperties' operation which provides
        additional details from classification object as well. This operation only provides classification class
        hierarchy details.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception. In all other cases
        failures will be returned with the WorkspaceObject ID mapped to the error message in the 'ServiceData' list of
        partial errors of the returned 'GetClassificationHierarchiesResponse' structure
        """
        return cls.execute_soa_method(
            method_name='getClassificationHierarchies',
            library='Internal-Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'wsos': wsos},
            response_cls=GetClassificationHierarchiesResponse,
        )

    @classmethod
    def getClassificationProperties(cls, wsos: List[WorkspaceObject]) -> GetClassificationPropertiesResponse:
        """
        Gets information about classification objects based on workspace objects. A classification object is also
        called an ICO. ICO property values along with the underlying class hierarchy information can be retrieved. The
        hierarchy information provides details on the hierarchical structure of the classification class where the
        workspace object is classified.
        
        Use cases:
        User needs information about a classification object related to a workspace object. Information may be limited
        to the classification properties associated with the classification object and their values. Or it may include
        information on the underlying classification class structure for the classification object
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getClassificationProperties',
            library='Internal-Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'wsos': wsos},
            response_cls=GetClassificationPropertiesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Rdv._2008_03.VariantManagement import BomlinesInfoForValidoverlay, GetValidoverlayBomlineResponse
from typing import List
from tcsoa.utils import TcService


class VariantManagementService(TcService):

    @classmethod
    def getValidoverlayBomlinesInfo(cls, input: List[BomlinesInfoForValidoverlay]) -> GetValidoverlayBomlineResponse:
        """
        This method performs Valid Overlays Only analysis on the background BOMLine objects provided to it, based on
        the set of VariantRule objects provided by the caller. It matches all the BOMLine objects against each of the
        VariantRule provided and returns them classified as configured and uncofigured BOMLine objects. It also
        performs further filtering based on the effectivity date, based on the caller provided boolean flag. The caller
        can either invoke both the filters or choose to invoke any one of them, by appropriately setting the input flag
        as true.
        The method does not accept any context information and assumes that the BOMLine objects and the VariantRule
        objects are from the same context.
        
        
        Use cases:
        Use case 1: Applying Valid Overlays Only (VOO) filter on the result of a search
        If the user wishes to filter any particular set of BOMLine objects using VOO filter, then this operation can be
        invoked. This operation allows the caller to independantly process any set of BOMLine objects against any set
        of VariantRule object(s). To skip the date filter, user would have to set the 'showUnconfiguredBydate' flag to
        true.
        
        Use case 2: Applying Effectivity Date filter on the result of a search
        If the user wishes to filter a set of BOMLine objects based on their effectivity dates, then this operation can
        be invoked by setting the showUnconfiguredVariant flag to true.
        """
        return cls.execute_soa_method(
            method_name='getValidoverlayBomlinesInfo',
            library='Internal-Rdv',
            service_date='2008_03',
            service_name='VariantManagement',
            params={'input': input},
            response_cls=GetValidoverlayBomlineResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine, CAEItemRevision
from tcsoa.gen.Internal.Cae._2011_06.StructureManagement import ExecuteRuleResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def updateModelAttsByDM(cls, productBOMLine: BOMLine, modelParentBOMLine: BOMLine, modelRevTag: ItemRevision, modelExistingBOMLine: BOMLine, domain: str, bomlineAttribsOnly: bool, doRevise: bool) -> ExecuteRuleResponse:
        """
        This internal operation will update the attributes of the given CAEModel Revision as per attributes of the
        given product BOMLine, after applying data mapping rules to it. The data mapping rules will be applied as per
        the given domain.
        """
        return cls.execute_soa_method(
            method_name='updateModelAttsByDM',
            library='Internal-Cae',
            service_date='2012_09',
            service_name='StructureManagement',
            params={'productBOMLine': productBOMLine, 'modelParentBOMLine': modelParentBOMLine, 'modelRevTag': modelRevTag, 'modelExistingBOMLine': modelExistingBOMLine, 'domain': domain, 'bomlineAttribsOnly': bomlineAttribsOnly, 'doRevise': doRevise},
            response_cls=ExecuteRuleResponse,
        )

    @classmethod
    def createNewModelByDM(cls, productBOMLine: BOMLine, modelParentBOMLine: BOMLine, modelExistingBOMLine: BOMLine, domain: str) -> ExecuteRuleResponse:
        """
        This internal operation will create a CAEModel Item Revision by applying Data Map rules to the given product
        bomline. The data mapping rules will be applied as per the given domain.
        """
        return cls.execute_soa_method(
            method_name='createNewModelByDM',
            library='Internal-Cae',
            service_date='2012_09',
            service_name='StructureManagement',
            params={'productBOMLine': productBOMLine, 'modelParentBOMLine': modelParentBOMLine, 'modelExistingBOMLine': modelExistingBOMLine, 'domain': domain},
            response_cls=ExecuteRuleResponse,
        )

    @classmethod
    def executeDatamap(cls, rootBOMLine: BOMLine, domain: str) -> ExecuteRuleResponse:
        """
        This internal operation creates an output BOM structure given the root BOMLine of an input BOM structure loaded
        in a BOMWindow. The output BOM structure is determined by the XSLT-based Data Map rules executed against the
        input BOM structure. Data Map syntax is in compliance with the schema defined in tcsim_xslWithNode.xsd, located
        in TC_DATA.
        Data Map rules define the mapping between an input item type and its resulting output item type. Data Map rules
        are defined for an entire site and are stored in the datamapping.xml file located in TC_DATA. The naming of the
        data mapping file is defined by the site preference CAE_dataMapping_file.
        The datamapping.xml file can be configured for various domains defined as LOV objects under StructureMap
        Domains in BMIDE. To configure the domains, in the Extensions view in BMIDE, open LOV->StructureMap Domains and
        add additional domain values. This operation assumes the value of the domain as CAE.
        To use this operation, a well-defined datamapping.xml is required in TC_DATA.
        """
        return cls.execute_soa_method(
            method_name='executeDatamap',
            library='Internal-Cae',
            service_date='2012_09',
            service_name='StructureManagement',
            params={'rootBOMLine': rootBOMLine, 'domain': domain},
            response_cls=ExecuteRuleResponse,
        )

    @classmethod
    def executeMarkUpToDate(cls, inputItemRevList: List[CAEItemRevision]) -> ServiceData:
        """
        This internal operation will check if there exists any status object associated with the input CAEItem
        Revisions. If it exists it will update the attributes of the status object. If no status object found to be
        associated with the CAEItem Revision, a new object will be created
        """
        return cls.execute_soa_method(
            method_name='executeMarkUpToDate',
            library='Internal-Cae',
            service_date='2012_09',
            service_name='StructureManagement',
            params={'inputItemRevList': inputItemRevList},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Multisite._2018_06.Search import FetchOdsResponse, StringVectorMap
from typing import List
from tcsoa.utils import TcService


class SearchService(TcService):

    @classmethod
    def fetchOdsRecords(cls, odsSiteName: str, searchMode: int, startDate: str, endDate: str, pubrList: List[str], odsSessionInfo: StringVectorMap) -> FetchOdsResponse:
        """
        This operation queries for Object Directory Service (ODS) publication records at the target ODS site based on
        the input criteria and serializes the data into a TCXML file. The search criteria can be date based or a list
        of objects which have been previously published. Read tickets are returned to the generated files.
        
        Use cases:
        This will be used to extract ODS data for indexing purposes. The output will be used to populate a seache
        engine like Solr.
        """
        return cls.execute_soa_method(
            method_name='fetchOdsRecords',
            library='Internal-Multisite',
            service_date='2018_06',
            service_name='Search',
            params={'odsSiteName': odsSiteName, 'searchMode': searchMode, 'startDate': startDate, 'endDate': endDate, 'pubrList': pubrList, 'odsSessionInfo': odsSessionInfo},
            response_cls=FetchOdsResponse,
        )



from tcsoa.gen.Internal_Reports._2007_06.services import BOMRollupService as imp0
from tcsoa.utils import TcService


class BOMRollupService(TcService):
    cloneRollupTemplates = imp0.cloneRollupTemplates
    createRollupCalculationTemplates = imp0.createRollupCalculationTemplates
    createRollupTemplates = imp0.createRollupTemplates
    generateRollupReports = imp0.generateRollupReports
    getRollupTemplateCalculations = imp0.getRollupTemplateCalculations
    getRollupTemplates = imp0.getRollupTemplates
    reviseRollupReports = imp0.reviseRollupReports



from __future__ import annotations

from tcsoa.gen.Visualization._2013_12.StructureManagement import StringKeyToStringVectorMap, CreateVisSCsFromBOMsInfo
from tcsoa.gen.Visualization._2011_02.StructureManagement import CreateVisSCsFromBOMsResponse
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def createVisSCsFromBOMsWithOptions(cls, info: List[CreateVisSCsFromBOMsInfo], globalOptions: StringKeyToStringVectorMap) -> CreateVisSCsFromBOMsResponse:
        """
        This operation takes a list of BOMLines (the occurrences list) and returns the VisStructureContext objects
        representing the configuration state of the BOMWindow (referred to as the configuration recipe). This
        configuration includes:
        - The occurrence UID chains for the input/selected BOMLines up to but not including the top line. 
        - Optional IMANFile reference to the PLMXML static representation of the BOMWindow.
        
        
        
        This service supports both the interoperation of selected BOMLines from the Teamcenter Rich Client to
        Teamcenter Visualization and also the capture/persistence of the configuration recipe for a particular
        BOMWindow. The occurrence list records the selected BOMLines at the time of interoperation and can be used in
        later operations to populate/expand a BOMWindow with those same occurrences.
        
        Use cases:
        When the user desires to create a persistent object that records the configuration recipe of a particular
        BOMWindow. The resulting VisStructureContext object would assumedly be used to later reconstruct a BOMWindow
        with the same configuration recipe and the recorded UID occurrence chains would be used to populate/expand the
        constructed BOMWindow with specific BOMLines. For example, this operation is used when sending selected
        BOMLines from the Structure Manager to Teamcenter Visualization and also to capture the configuration recipe
        for storage in Vis Sessions.
        
        Visualization pruned launch use case
        - User opens a structure in Structure Manager (SM)/Multi Structure Manager (MSM)/Manufacturing Process Planner
        (MPP), and configures it
        - User selects some lines they want to send to visualization as a pruned structure
        - System calls createVisSCsFromBOMsWithOptions to record the selected lines and the configuration of the BOM to
        send
        
        
        
        Visualization session save use case
        - 1.    User performs Visualization pruned launch use case and loads occurrences into visualization
        - 2.    User creates some authored visualization content (e.g. snapshots, motions, etc)
        - 3.    User saves session to Teamcenter
        - 4.    System calls createRecipesFromBOMs operation to capture the configuration and any pruning information
        as a VisStructureContext object.  UID of object returned.
        - 5.    System writes the VisStructureContext object reference into the visualization session data
        - 6.    System saves the visualiation session dataset to Teamcenter, and relates it to the VisStructureContext
        object created by the service
        
        
        
        Visualization Technical Illustration and 3D Markup save use cases
        Similar to session save use case, except saving a different data type.  Uses this service to create the recipe
        for the authored visualization data in the Teamcenter data model.
        
        Use Case Dependencies: 
        The createVisSCsFromBOMsWithOptions operation is called with input BOMLines from an existing BOM Window.
        Therefore, the BOMWindow must have already been created and populated with at least a top line.
        """
        return cls.execute_soa_method(
            method_name='createVisSCsFromBOMsWithOptions',
            library='Visualization',
            service_date='2013_12',
            service_name='StructureManagement',
            params={'info': info, 'globalOptions': globalOptions},
            response_cls=CreateVisSCsFromBOMsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import CadAttrMappingDefinition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAttrMappingsFilter(TcBaseObj):
    """
    Input filter for the data returned from the 'getAttrMappings' operation.
    
    :var itemTypeName: The item type name used to find an attribute mapping.
    :var datasetTypeName: The dataset type name used to find an attribute mapping.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure. If 'clientId' is not provided then it can be difficult to align the
    input with any returned errors.
    """
    itemTypeName: str = ''
    datasetTypeName: str = ''
    clientId: str = ''


@dataclass
class GetAttrMappingsResponse(TcBaseObj):
    """
    The response returned from the 'getAttrMappings' operation.
    
    :var attrMappingInfos: A list of attribute mapping information.
    :var serviceData: The 'ServiceData'. This operation will populate the ServiceData plain objects with
    CadAttrMappingDefinition objects and property descriptor LOV objects.
    """
    attrMappingInfos: List[AttrMappingInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AttrMappingInfo(TcBaseObj):
    """
    Attribute mapping information.
    
    :var cadAttrMappingDefinition: The CadAttrMappingDefinition object reference representing the mapping definition.
    :var propDescInfo: The property descriptor information for the attribute mapping.  This information can be used to
    look up the client-side business object 'Type' and 'PropertyDescription' object in the 'ClientMetaModel' data.
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure. If 'clientId' is not provided then it can be difficult to align the
    input with any returned errors.
    """
    cadAttrMappingDefinition: CadAttrMappingDefinition = None
    propDescInfo: PropDescInfo = None
    clientId: str = ''


@dataclass
class PropDescInfo(TcBaseObj):
    """
    The 'PropDescInfo' struct contains information about the Teamcenter property descriptor associated with the
    attribute mapping definition.
    
    :var boTypeName: The business object type name with which the property descriptor is associated.  This can be used
    to look up the client-side 'PropertyDescription' object in the SOA 'ClientMetaModel' data.
    :var propDescName: The name of the property descriptor.  This can be used to look up the client-side
    'PropertyDescription' object in the 'ClientMetaModel' data.
    """
    boTypeName: str = ''
    propDescName: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanQuery
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindSavedQueriesCriteriaInput(TcBaseObj):
    """
    Structure that contains the input criteria required to find the saved queries.
    
    :var queryNames: Names of saved queries to be found.
    :var queryDescs: Descrptions of saved queries to be found.
    :var queryType: The type of the saved queries.
    """
    queryNames: List[str] = ()
    queryDescs: List[str] = ()
    queryType: int = 0


@dataclass
class FindSavedQueriesResponse(TcBaseObj):
    """
    Holds the response for FindSavedQueries.
    
    :var savedQueries: A vector of Saved Query objects.
    :var serviceData: Standard ServiceData
    """
    savedQueries: List[ImanQuery] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Manufacturing._2012_02.DataManagement import AssociateAndAllocateInput, ConnectObjectsInputData, GetAssociatedContextsInputData, AddAssociationInput, DisconnectFromOriginInputData, AutomaticAllocatePreviewResponse, ConnectObjectResponse, AssociateAndAllocateResponse, AllocationMap, AssociationResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2012_02.Model import ToolRequirementInput, ResolveDataInfo, CandidateToolsForToolRequirement, ToolRequirementResponse
from tcsoa.gen.Manufacturing._2012_02.IPAManagement import DeleteFilteredIPAInputInfo, GetFilteredIPATypeResponse
from tcsoa.gen.Manufacturing._2012_02.Constraints import GetPrecedenceConstraintPathsInputs, ValidateConstraintConsistencyResponse, GetPrecedenceConstraintsResponse, ValidateProcessAreaAssignmentsResponse, ValidateProcessAreaAssignmentsInputs, GetPrecedenceConstraintPathsResponse, ValidateConstraintConsistencyInputs, GetPrecedenceConstraintsIn
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def addAssociatedContexts(cls, input: List[AddAssociationInput]) -> ServiceData:
        """
        Generic operation for setting different kinds of associations. It could be between Plant BOP and Product BOP
        (Leading Plant BOP in the future), Product (root) and Process (root) structures, regular Plant (Workarea)
        (root) and Process (root) structures, EBOM-MBOM. For now we will use it as threshold for Plant BOP and Product
        BOP. Other types are defined as target.
        This SOA will be used for add operation. Remove/disconnect (future) will be implemented in separate SOA.
        """
        return cls.execute_soa_method(
            method_name='addAssociatedContexts',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def connectObjects(cls, input: List[ConnectObjectsInputData]) -> ConnectObjectResponse:
        """
        Generic operation for connecting MFG nodes      
        int        quantityNum - number of BOMLines to create (used in paste special). Default value is 1.
        int        occurrenceNumber - for packed lines. Number of occurrences to create (used in paste special).
        Default value is 1.
        int        findNumber - find number. Always passed by client.
        """
        return cls.execute_soa_method(
            method_name='connectObjects',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ConnectObjectResponse,
        )

    @classmethod
    def disconnectFromOrigin(cls, inputVector: List[DisconnectFromOriginInputData]) -> ServiceData:
        """
        Disconnects objects and their children from their origin relation. Origin relation is created when the objects
        are assigned from the Product BOP/Generic BOP to the Plant BOP/Product BOP. This functionality removes the
        origin relation created during the assignment. Also this functionality can be called recursively for the
        processes below. This functionality can return an error in the following conditions, the object on which
        disconnect function was called called does not have an origin in case of non-recursive calls. This error will
        have a severity level of information. And This functionality is not available on this type of object only
        ProcessAreas, WorkAreas, Partitions, Processes and Operations type objects are expected.
        """
        return cls.execute_soa_method(
            method_name='disconnectFromOrigin',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'inputVector': inputVector},
            response_cls=ServiceData,
        )

    @classmethod
    def associateAndAllocateByPreview(cls, input: AssociateAndAllocateInput, allocationMap: AllocationMap) -> AssociateAndAllocateResponse:
        """
        This function is the second function call in case a preview is required for automatic allocation this function
        does the actual allocation from Source Product BOP to a Target Plant BOP on the basis of a Reference Product
        BOP. This is done by passing the allocation map back to the server which we recived in the 
        automaticAllocatePreview command. This function also associates the Source Product BOP with the Target Plant
        BOP in case they are not associated. This function can throw the following exceptions, Reference Product BOP is
        not linked to the Target Plant BOP and Some allocation from the source structure to the target structure were
        unsuccessful please see the log for more details. Both these error messages will have the severity level of
        error.
        """
        return cls.execute_soa_method(
            method_name='associateAndAllocateByPreview',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input, 'allocationMap': allocationMap},
            response_cls=AssociateAndAllocateResponse,
        )

    @classmethod
    def getAssociatedContexts(cls, input: List[GetAssociatedContextsInputData]) -> AssociationResponse:
        """
        Returns associated contexts with the input context
        """
        return cls.execute_soa_method(
            method_name='getAssociatedContexts',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=AssociationResponse,
        )

    @classmethod
    def automaticAllocatePreview(cls, input: AssociateAndAllocateInput) -> AutomaticAllocatePreviewResponse:
        """
        This function is the first function call in case a preview is required for automatic allocation from Source
        Product BOP to a Target Plant BOP on the basis of a Reference Product BOP. This function finds the allocated
        lines from the reference product BOP to the Plant BOP and equivalent lines in the source Product BOP and
        generates a preview for the automatic allocation in CSV format . Also this function returns an allocationMap,
        which needs to be sent back to server in case the user wants to go ahead with the allocation , for which it
        calls the second server call associateAndAllocateMap command . This function can throw the following
        exceptions, Reference Product BOP is not linked to the Target Plant BOP . This error messages will have
        severity level of error.
        """
        return cls.execute_soa_method(
            method_name='automaticAllocatePreview',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=AutomaticAllocatePreviewResponse,
        )

    @classmethod
    def automaticAssociateAndAllocate(cls, input: AssociateAndAllocateInput) -> AssociateAndAllocateResponse:
        """
        This function is a single call function that does the allocation from Source Product BOP to a Target Plant BOP
        on the basis of a Reference Product BOP. This function finds the allocated lines from the reference product BOP
        to the Plant BOP and equivalent lines in the source Product BOP and does the allocation from the Source Product
        BOP to the Target Plant BOP in a single call. This function also associates the Source Product BOP with the
        Target Plant BOP in case they are not associated. This function can throw the following exceptions, Reference
        Product BOP is not linked to the Target Plant BOP and Some allocation from the source structure to the target
        structure were unsuccessful please see the log for more details. Both these error messages will have the
        severity level of error.
        """
        return cls.execute_soa_method(
            method_name='automaticAssociateAndAllocate',
            library='Manufacturing',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input},
            response_cls=AssociateAndAllocateResponse,
        )


class ModelService(TcService):

    @classmethod
    def getToolRequirements(cls, parentObject: List[BusinessObject], toolRequirementStatus: List[str]) -> ToolRequirementResponse:
        """
        Fetches the Tool Requirement for the given operation or process. Based on the status of the Tool Requirement.
        either all. resolved or unresolved Tool Requirements are returned. The respective options for the status are
        ALL. RESOLVED and UNRESOLVED.
        Note that the Tool requirement assigned to child operation or process is not considered.
        """
        return cls.execute_soa_method(
            method_name='getToolRequirements',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Model',
            params={'parentObject': parentObject, 'toolRequirementStatus': toolRequirementStatus},
            response_cls=ToolRequirementResponse,
        )

    @classmethod
    def resolveToolRequirement(cls, resolveObjects: List[ResolveDataInfo]) -> ServiceData:
        """
        Resolves the Tool Requirement with the provided tool. As a result the Tool is assigned to the Tool Requirement
        and to the Operation for which Tool Requirement is defined. This resolve operation  is allowed only in the
        Plant BOP.
        """
        return cls.execute_soa_method(
            method_name='resolveToolRequirement',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Model',
            params={'resolveObjects': resolveObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def updateFlows(cls, input: List[BusinessObject], isSubHierarchies: bool) -> ServiceData:
        """
        Updates flows between the children of input object(s) based on Find number value. Input objects should be an
        instance of BOM line. This service does not return any resulting or affected objects. The client needs to
        update cache of affected objects manually(children of the input object are affected ). If isSubHierarchies
        parameter is true, flows are recursively updated for all children.
        """
        return cls.execute_soa_method(
            method_name='updateFlows',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Model',
            params={'input': input, 'isSubHierarchies': isSubHierarchies},
            response_cls=ServiceData,
        )

    @classmethod
    def getCandidateToolsForToolRequirement(cls, resolveObjects: List[ToolRequirementInput]) -> CandidateToolsForToolRequirement:
        """
        Gets the candidate tools against which Tool Requirement could be resolved. The candidate tools are fetched
        based on the search criteria specified on the Tool Requirement. The input parameter tool source specifies the
        source from where candidate tools are supposed to be fetched.  The candidate tool can be fetched only in the
        Plant BOP.
        """
        return cls.execute_soa_method(
            method_name='getCandidateToolsForToolRequirement',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Model',
            params={'resolveObjects': resolveObjects},
            response_cls=CandidateToolsForToolRequirement,
        )


class ConstraintsService(TcService):

    @classmethod
    def validateConstraintConsistency(cls, inputData: List[ValidateConstraintConsistencyInputs]) -> ValidateConstraintConsistencyResponse:
        """
        This SOA invokes the consistency check service. It checks whether the constraints defined in the product BOP or
        a sub structure thereof are consistent.
        """
        return cls.execute_soa_method(
            method_name='validateConstraintConsistency',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Constraints',
            params={'inputData': inputData},
            response_cls=ValidateConstraintConsistencyResponse,
        )

    @classmethod
    def validateProcessAreaAssignments(cls, inputData: List[ValidateProcessAreaAssignmentsInputs]) -> ValidateProcessAreaAssignmentsResponse:
        """
        This SOA invokes the constraint validation service. It checks whether the assignments of operations or
        processes of a Product BOP to process areas in a Plant BOP are consistent with the constraint definitions.
        """
        return cls.execute_soa_method(
            method_name='validateProcessAreaAssignments',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Constraints',
            params={'inputData': inputData},
            response_cls=ValidateProcessAreaAssignmentsResponse,
        )

    @classmethod
    def getPrecedenceConstraintPaths(cls, inputData: List[GetPrecedenceConstraintPathsInputs]) -> GetPrecedenceConstraintPathsResponse:
        """
        Returns all operations/processes in the precedence chain starting from the given start operation/process up to
        the end operation/process, i.e., all operations/processes succeeding the start operation/process and preceding
        the end operation/process.
        """
        return cls.execute_soa_method(
            method_name='getPrecedenceConstraintPaths',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Constraints',
            params={'inputData': inputData},
            response_cls=GetPrecedenceConstraintPathsResponse,
        )

    @classmethod
    def getPrecedenceConstraints(cls, inputData: List[GetPrecedenceConstraintsIn]) -> GetPrecedenceConstraintsResponse:
        """
        Fetches all precedence constraints defined on the given input objects - traverses 'm' levels in the predecessor
        chain and 'n' levels in the successor chain
        """
        return cls.execute_soa_method(
            method_name='getPrecedenceConstraints',
            library='Manufacturing',
            service_date='2012_02',
            service_name='Constraints',
            params={'inputData': inputData},
            response_cls=GetPrecedenceConstraintsResponse,
        )


class IPAManagementService(TcService):

    @classmethod
    def deleteFilteredIPA(cls, input: List[DeleteFilteredIPAInputInfo]) -> ServiceData:
        """
        This API will delete the filtered IPA(s) under the process depending on the boolean member "isRecursive" of the
        input structure.
        If " isRecursive" is true, then all the filtered IPAs in the hierarchy of the member "process" will be deleted.
        Else just one filtered IPA directly under the process will be deleted.
        """
        return cls.execute_soa_method(
            method_name='deleteFilteredIPA',
            library='Manufacturing',
            service_date='2012_02',
            service_name='IPAManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getFilteredIPAType(cls, processes: List[BusinessObject]) -> GetFilteredIPATypeResponse:
        """
        For each process, return the type of the FIPA used for this process structure. 
        For processes from the same process structure, the answer is the same. Therefore, from perforemence point of
        view, it is better to pass the process context (topline) as an input, instead of sending several processes from
        the same structure.
        """
        return cls.execute_soa_method(
            method_name='getFilteredIPAType',
            library='Manufacturing',
            service_date='2012_02',
            service_name='IPAManagement',
            params={'processes': processes},
            response_cls=GetFilteredIPATypeResponse,
        )
